@@ -1,27 +1,27 @@
 /*
-    Copyright (c) 2013-2019 Cong Xu
-    All rights reserved.
+	Copyright (c) 2013-2023 Cong Xu
+	All rights reserved.
 
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
+	Redistribution and use in source and binary forms, with or without
+	modification, are permitted provided that the following conditions are met:
 
-    Redistributions of source code must retain the above copyright notice, this
-    list of conditions and the following disclaimer.
-    Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
+	Redistributions of source code must retain the above copyright notice, this
+	list of conditions and the following disclaimer.
+	Redistributions in binary form must reproduce the above copyright notice,
+	this list of conditions and the following disclaimer in the documentation
+	and/or other materials provided with the distribution.
 
-    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-    ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
+	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+	POSSIBILITY OF SUCH DAMAGE.
 */
 #include "briefing_screens.h"
 
@@ -31,25 +31,28 @@
 #include <cdogs/files.h>
 #include <cdogs/font.h>
 #include <cdogs/grafx_bg.h>
+#include <cdogs/log.h>
 #include <cdogs/music.h>
 #include <cdogs/objective.h>
 
 #include "animated_counter.h"
 #include "autosave.h"
 #include "hiscores.h"
+#include "loading_screens.h"
 #include "menu_utils.h"
 #include "password.h"
 #include "prep.h"
 #include "prep_equip.h"
 #include "screens_end.h"
 
-
 static void DrawObjectiveInfo(const Objective *o, const struct vec2i pos);
 
 typedef struct
 {
 	EventWaitResult waitResult;
-	const CampaignSetting *c;
+	CampaignSetting *c;
+	GameMode gameMode;
+	const CampaignEntry *entry;
 } ScreenCampaignIntroData;
 static void CampaignIntroTerminate(GameLoopData *data);
 static void CampaignIntroOnEnter(GameLoopData *data);
@@ -57,14 +60,18 @@ static void CampaignIntroOnExit(GameLoopData *data);
 static void CampaignIntroInput(GameLoopData *data);
 static GameLoopResult CampaignIntroUpdate(GameLoopData *data, LoopRunner *l);
 static void CampaignIntroDraw(GameLoopData *data);
-GameLoopData *ScreenCampaignIntro(CampaignSetting *c)
+GameLoopData *ScreenCampaignIntro(
+	CampaignSetting *c, const GameMode gameMode, const CampaignEntry *entry)
 {
 	ScreenCampaignIntroData *data;
 	CMALLOC(data, sizeof *data);
 	data->c = c;
+	data->gameMode = gameMode;
+	data->entry = entry;
 	return GameLoopDataNew(
-		data, CampaignIntroTerminate, CampaignIntroOnEnter, CampaignIntroOnExit,
-		CampaignIntroInput, CampaignIntroUpdate, CampaignIntroDraw);
+		data, CampaignIntroTerminate, CampaignIntroOnEnter,
+		CampaignIntroOnExit, CampaignIntroInput, CampaignIntroUpdate,
+		CampaignIntroDraw);
 }
 static void CampaignIntroTerminate(GameLoopData *data)
 {
@@ -73,17 +80,24 @@ static void CampaignIntroTerminate(GameLoopData *data)
 }
 static void CampaignIntroOnEnter(GameLoopData *data)
 {
-	UNUSED(data);
-	MusicPlay(&gSoundDevice, MUSIC_BRIEFING, NULL, NULL);
+	ScreenCampaignIntroData *sData = data->Data;
+	gCampaign.Entry.Mode = sData->gameMode;
+	if (!gCampaign.IsLoaded && !CampaignLoad(&gCampaign, sData->entry))
+	{
+		// Failed to load
+		LOG(LM_MAIN, LL_ERROR, "cannot load campaign %s", sData->entry->Info);
+	}
+	else
+	{
+		MusicPlayFromChunk(
+			&gSoundDevice.music, MUSIC_MENU,
+			&sData->c->CustomSongs[MUSIC_MENU]);
+	}
 }
 static void CampaignIntroOnExit(GameLoopData *data)
 {
 	const ScreenCampaignIntroData *sData = data->Data;
-	if (sData->waitResult != EVENT_WAIT_CANCEL)
-	{
-		SoundPlay(&gSoundDevice, StrSound("mg"));
-	}
-	else
+	if (sData->waitResult == EVENT_WAIT_CANCEL)
 	{
 		CampaignUnload(&gCampaign);
 	}
@@ -92,21 +106,29 @@ static void CampaignIntroInput(GameLoopData *data)
 {
 	ScreenCampaignIntroData *sData = data->Data;
 	sData->waitResult = EventWaitForAnyKeyOrButton();
+	if (sData->waitResult == EVENT_WAIT_OK)
+	{
+		MenuPlaySound(MENU_SOUND_ENTER);
+	}
 }
 static GameLoopResult CampaignIntroUpdate(GameLoopData *data, LoopRunner *l)
 {
 	const ScreenCampaignIntroData *sData = data->Data;
 
-	if (!IsIntroNeeded(gCampaign.Entry.Mode) ||
+	if (!gCampaign.IsLoaded || sData->waitResult == EVENT_WAIT_CANCEL)
+	{
+		LoopRunnerPop(l);
+	}
+	else if (
+		!IsIntroNeeded(gCampaign.Entry.Mode) ||
 		sData->waitResult == EVENT_WAIT_OK)
 	{
 		// Switch to num players selection
-		LoopRunnerChange(
-			l, NumPlayersSelection(&gGraphicsDevice, &gEventHandlers));
-	}
-	else if (sData->waitResult == EVENT_WAIT_CANCEL)
-	{
-		LoopRunnerPop(l);
+		LoopRunnerPush(
+			l,
+			ScreenLoading(
+				"Loading campaign...", true,
+				NumPlayersSelection(&gGraphicsDevice, &gEventHandlers), true));
 	}
 	return UPDATE_RESULT_OK;
 }
@@ -144,7 +166,6 @@ static void CampaignIntroDraw(GameLoopData *data)
 	BlitUpdateFromBuf(&gGraphicsDevice, gGraphicsDevice.screen);
 }
 
-
 typedef struct
 {
 	char *Title;
@@ -158,15 +179,18 @@ typedef struct
 	struct vec2i ObjectiveDescPos;
 	struct vec2i ObjectiveInfoPos;
 	int ObjectiveHeight;
+	CampaignSetting *C;
 	const struct MissionOptions *MissionOptions;
 	EventWaitResult waitResult;
 } MissionBriefingData;
 static void MissionBriefingTerminate(GameLoopData *data);
+static void MissionBriefingOnEnter(GameLoopData *data);
 static void MissionBriefingOnExit(GameLoopData *data);
 static void MissionBriefingInput(GameLoopData *data);
 static GameLoopResult MissionBriefingUpdate(GameLoopData *data, LoopRunner *l);
 static void MissionBriefingDraw(GameLoopData *data);
-GameLoopData *ScreenMissionBriefing(const struct MissionOptions *m)
+GameLoopData *ScreenMissionBriefing(
+	CampaignSetting *c, const struct MissionOptions *m)
 {
 	const int w = gGraphicsDevice.cachedConfig.Res.x;
 	const int h = gGraphicsDevice.cachedConfig.Res.y;
@@ -179,25 +203,15 @@ GameLoopData *ScreenMissionBriefing(const struct MissionOptions *m)
 	if (m->missionData->Title)
 	{
 		CMALLOC(mData->Title, strlen(m->missionData->Title) + 32);
-		sprintf(mData->Title, "Mission %d: %s",
-			m->index + 1, m->missionData->Title);
+		sprintf(
+			mData->Title, "Mission %d: %s", m->index + 1,
+			m->missionData->Title);
 		mData->TitleOpts = FontOptsNew();
 		mData->TitleOpts.HAlign = ALIGN_CENTER;
 		mData->TitleOpts.Area = gGraphicsDevice.cachedConfig.Res;
 		mData->TitleOpts.Pad.y = y - 25;
 	}
 
-	// Password
-	if (m->index > 0)
-	{
-		sprintf(
-			mData->Password, "Password: %s", gAutosave.LastMission.Password);
-		mData->PasswordOpts = FontOptsNew();
-		mData->PasswordOpts.HAlign = ALIGN_CENTER;
-		mData->PasswordOpts.Area = gGraphicsDevice.cachedConfig.Res;
-		mData->PasswordOpts.Pad.y = y - 15;
-	}
-
 	// Description
 	if (m->missionData->Description)
 	{
@@ -219,11 +233,13 @@ GameLoopData *ScreenMissionBriefing(const struct MissionOptions *m)
 			svec2i(w - (w / 6), mData->ObjectiveDescPos.y + FontH());
 		mData->ObjectiveHeight = h / 12;
 	}
+	mData->C = c;
 	mData->MissionOptions = m;
 
 	return GameLoopDataNew(
-		mData, MissionBriefingTerminate, NULL, MissionBriefingOnExit,
-		MissionBriefingInput, MissionBriefingUpdate, MissionBriefingDraw);
+		mData, MissionBriefingTerminate, MissionBriefingOnEnter,
+		MissionBriefingOnExit, MissionBriefingInput, MissionBriefingUpdate,
+		MissionBriefingDraw);
 }
 static void MissionBriefingTerminate(GameLoopData *data)
 {
@@ -234,15 +250,21 @@ static void MissionBriefingTerminate(GameLoopData *data)
 	CFREE(mData->TypewriterBuf);
 	CFREE(mData);
 }
+static void MissionBriefingOnEnter(GameLoopData *data)
+{
+	MissionBriefingData *mData = data->Data;
+	if (IsMissionBriefingNeeded(gCampaign.Entry.Mode, mData->Description))
+	{
+		MusicPlayFromChunk(
+			&gSoundDevice.music, MUSIC_BRIEFING,
+			&mData->C->CustomSongs[MUSIC_BRIEFING]);
+	}
+}
 static void MissionBriefingOnExit(GameLoopData *data)
 {
 	const MissionBriefingData *mData = data->Data;
 
-	if (mData->waitResult == EVENT_WAIT_OK)
-	{
-		SoundPlay(&gSoundDevice, StrSound("mg"));
-	}
-	else
+	if (mData->waitResult != EVENT_WAIT_OK)
 	{
 		CampaignUnload(&gCampaign);
 	}
@@ -270,11 +292,13 @@ static void MissionBriefingInput(GameLoopData *data)
 				}
 				// Otherwise, exit out of loop
 				mData->waitResult = EVENT_WAIT_OK;
+				MenuPlaySound(MENU_SOUND_ENTER);
 			}
 		}
 	}
 	// Check if anyone pressed escape
-	if (EventIsEscape(&gEventHandlers, cmds, GetMenuCmd(&gEventHandlers)))
+	if (EventIsEscape(
+			&gEventHandlers, cmds, GetMenuCmd(&gEventHandlers, false)))
 	{
 		mData->waitResult = EVENT_WAIT_CANCEL;
 	}
@@ -283,7 +307,7 @@ static GameLoopResult MissionBriefingUpdate(GameLoopData *data, LoopRunner *l)
 {
 	MissionBriefingData *mData = data->Data;
 
-	if (!IsMissionBriefingNeeded(gCampaign.Entry.Mode))
+	if (!IsMissionBriefingNeeded(gCampaign.Entry.Mode, mData->Description))
 	{
 		mData->waitResult = EVENT_WAIT_OK;
 		goto bail;
@@ -304,6 +328,13 @@ static GameLoopResult MissionBriefingUpdate(GameLoopData *data, LoopRunner *l)
 		return UPDATE_RESULT_DRAW;
 	}
 
+	// Auto skip if on demo mode
+	if (gEventHandlers.DemoQuitTimer > 0)
+	{
+		mData->waitResult = EVENT_WAIT_OK;
+		goto bail;
+	}
+
 	return UPDATE_RESULT_OK;
 
 bail:
@@ -332,17 +363,17 @@ static void MissionBriefingDraw(GameLoopData *data)
 	// Display objectives
 	CA_FOREACH(
 		const Objective, o, mData->MissionOptions->missionData->Objectives)
-		// Do not brief optional objectives
-		if (o->Required == 0)
-		{
-			continue;
-		}
-		struct vec2i offset = svec2i(0, _ca_index * mData->ObjectiveHeight);
-		FontStr(o->Description, svec2i_add(mData->ObjectiveDescPos, offset));
-		// Draw the icons slightly offset so that tall icons don't overlap each
-		// other
-		offset.x = -16 * (_ca_index & 1);
-		DrawObjectiveInfo(o, svec2i_add(mData->ObjectiveInfoPos, offset));
+	// Do not brief optional objectives
+	if (o->Required == 0)
+	{
+		continue;
+	}
+	struct vec2i offset = svec2i(0, _ca_index * mData->ObjectiveHeight);
+	FontStr(o->Description, svec2i_add(mData->ObjectiveDescPos, offset));
+	// Draw the icons slightly offset so that tall icons don't overlap each
+	// other
+	offset.x = -16 * (_ca_index & 1);
+	DrawObjectiveInfo(o, svec2i_add(mData->ObjectiveInfoPos, offset));
 	CA_FOREACH_END()
 
 	BlitUpdateFromBuf(&gGraphicsDevice, gGraphicsDevice.screen);
@@ -355,13 +386,15 @@ typedef struct
 	const PlayerData *Pd;
 	AnimatedCounter Score;
 	AnimatedCounter Total;
+	AnimatedCounter Time;
+	AnimatedCounter TimeTotal;
 	AnimatedCounter HealthResurrection;
 	AnimatedCounter ButcherNinjaFriendly;
 } PlayerSummaryDrawData;
 typedef struct
 {
 	MenuSystem ms;
-	const CampaignOptions *c;
+	const Campaign *c;
 	struct MissionOptions *m;
 	bool completed;
 	AnimatedCounter AccessBonus;
@@ -373,10 +406,10 @@ static void MissionSummaryOnEnter(GameLoopData *data);
 static GameLoopResult MissionSummaryUpdate(GameLoopData *data, LoopRunner *l);
 static void MissionSummaryDraw(GameLoopData *data);
 static void MissionSummaryMenuDraw(
-	const menu_t *menu, GraphicsDevice *g,
-	const struct vec2i p, const struct vec2i size, const void *data);
+	const menu_t *menu, GraphicsDevice *g, const struct vec2i p,
+	const struct vec2i size, const void *data);
 GameLoopData *ScreenMissionSummary(
-	const CampaignOptions *c, struct MissionOptions *m, const bool completed)
+	const Campaign *c, struct MissionOptions *m, const bool completed)
 {
 	MissionSummaryData *mData;
 	CCALLOC(mData, sizeof *mData);
@@ -407,8 +440,8 @@ GameLoopData *ScreenMissionSummary(
 	mData->completed = completed;
 
 	return GameLoopDataNew(
-		mData, MissionSummaryTerminate, MissionSummaryOnEnter, NULL,
-		NULL, MissionSummaryUpdate, MissionSummaryDraw);
+		mData, MissionSummaryTerminate, MissionSummaryOnEnter, NULL, NULL,
+		MissionSummaryUpdate, MissionSummaryDraw);
 }
 static void MissionSummaryTerminate(GameLoopData *data)
 {
@@ -439,26 +472,6 @@ static void MissionSummaryOnEnter(GameLoopData *data)
 {
 	MissionSummaryData *mData = data->Data;
 
-	MusicPlay(&gSoundDevice, MUSIC_BRIEFING, NULL, NULL);
-
-	if (mData->completed && IsPasswordAllowed(mData->c->Entry.Mode))
-	{
-		// Save password
-		MissionSave ms;
-		MissionSaveInit(&ms);
-		ms.Campaign = mData->c->Entry;
-		// Don't make password for next level if there is none
-		int passwordIndex = mData->m->index + 1;
-		if (passwordIndex == mData->c->Entry.NumMissions)
-		{
-			passwordIndex--;
-		}
-		strcpy(ms.Password, MakePassword(passwordIndex, 0));
-		ms.MissionsCompleted = mData->m->index + 1;
-		AutosaveAddMission(&gAutosave, &ms);
-		AutosaveSave(&gAutosave, GetConfigFilePath(AUTOSAVE_FILE));
-	}
-
 	// Calculate bonus scores
 	// Bonuses only apply if at least one player has lived
 	const int accessBonus = GetAccessBonus(mData->m);
@@ -467,18 +480,46 @@ static void MissionSummaryOnEnter(GameLoopData *data)
 		int bonus = 0;
 		// Objective bonuses
 		CA_FOREACH(const Objective, o, mData->m->missionData->Objectives)
-			if (ObjectiveIsPerfect(o))
-			{
-				bonus += PERFECT_BONUS;
-			}
+		if (ObjectiveIsPerfect(o))
+		{
+			bonus += PERFECT_BONUS;
+		}
 		CA_FOREACH_END()
 		bonus += accessBonus;
 		bonus += GetTimeBonus(mData->m, NULL);
 
 		CA_FOREACH(PlayerData, p, gPlayerDatas)
-			ApplyBonuses(p, bonus);
+		ApplyBonuses(p, bonus);
+		p->Totals.TimeTicks += mData->m->time;
 		CA_FOREACH_END()
 	}
+	
+	if (mData->completed && CanLevelSelect(mData->c->Entry.Mode))
+	{
+		AutosaveAdd(
+			&gAutosave, &mData->c->Entry, mData->m->index,
+			mData->m->NextMission, &gPlayerDatas);
+		AutosaveSave(&gAutosave, GetConfigFilePath(AUTOSAVE_FILE));
+	}
+
+	// Skip menu
+	if (mData->m->missionData->SkipDebrief)
+	{
+		return;
+	}
+
+	if (mData->completed)
+	{
+		MusicPlayFromChunk(
+			&gSoundDevice.music, MUSIC_END,
+			&gCampaign.Setting.CustomSongs[MUSIC_END]);
+	}
+	else
+	{
+		MusicPlayFromChunk(
+			&gSoundDevice.music, MUSIC_LOSE,
+			&gCampaign.Setting.CustomSongs[MUSIC_LOSE]);
+	}
 
 	// Init mission bonuses
 	if (AreAnySurvived())
@@ -498,56 +539,91 @@ static void MissionSummaryOnEnter(GameLoopData *data)
 	// Init per-player summaries
 	int idx = 0;
 	CA_FOREACH(PlayerData, pd, gPlayerDatas)
-		if (!pd->IsLocal)
+	if (!pd->IsLocal)
+	{
+		continue;
+	}
+	mData->pDatas[idx].Pd = pd;
+	mData->pDatas[idx].Score = AnimatedCounterNew("Score: ", pd->Stats.Score);
+	mData->pDatas[idx].Total = AnimatedCounterNew("Total: ", pd->Totals.Score);
+	mData->pDatas[idx].Time =
+		AnimatedCounterNew("Time: ", mData->m->time / FPS_FRAMELIMIT);
+	mData->pDatas[idx].TimeTotal = AnimatedCounterNew(
+		"Total Time: ", pd->Totals.TimeTicks / FPS_FRAMELIMIT);
+	if (pd->survived)
+	{
+		const int healthBonus = GetHealthBonus(pd);
+		if (healthBonus != 0)
 		{
-			continue;
+			mData->pDatas[idx].HealthResurrection =
+				AnimatedCounterNew("Health bonus: ", healthBonus);
 		}
-		mData->pDatas[idx].Pd = pd;
-		mData->pDatas[idx].Score =
-			AnimatedCounterNew("Score: ", pd->Stats.Score);
-		mData->pDatas[idx].Total =
-			AnimatedCounterNew("Total: ", pd->Totals.Score);
-		if (pd->survived)
+		const int resurrectionFee = GetResurrectionFee(pd);
+		if (resurrectionFee != 0)
 		{
-			const int healthBonus = GetHealthBonus(pd);
-			if (healthBonus != 0)
-			{
-				mData->pDatas[idx].HealthResurrection = AnimatedCounterNew("Health bonus: ", healthBonus);
-			}
-			const int resurrectionFee = GetResurrectionFee(pd);
-			if (resurrectionFee != 0)
-			{
-				mData->pDatas[idx].HealthResurrection = AnimatedCounterNew("Resurrection fee: ", resurrectionFee);
-			}
+			mData->pDatas[idx].HealthResurrection =
+				AnimatedCounterNew("Resurrection fee: ", resurrectionFee);
+		}
 
-			const int butcherPenalty = GetButcherPenalty(pd);
-			if (butcherPenalty != 0)
-			{
-				mData->pDatas[idx].ButcherNinjaFriendly = AnimatedCounterNew("Butcher penalty: ", butcherPenalty);
-			}
-			const int ninjaBonus = GetNinjaBonus(pd);
-			if (ninjaBonus != 0)
-			{
-				mData->pDatas[idx].ButcherNinjaFriendly = AnimatedCounterNew("Ninja bonus: ", ninjaBonus);
-			}
-			const int friendlyBonus = GetFriendlyBonus(pd);
-			if (friendlyBonus != 0)
-			{
-				mData->pDatas[idx].ButcherNinjaFriendly = AnimatedCounterNew("Friendly bonus: ", friendlyBonus);
-			}
+		const int butcherPenalty = GetButcherPenalty(pd);
+		if (butcherPenalty != 0)
+		{
+			mData->pDatas[idx].ButcherNinjaFriendly =
+				AnimatedCounterNew("Butcher penalty: ", butcherPenalty);
 		}
-		idx++;
+		const int ninjaBonus = GetNinjaBonus(pd);
+		if (ninjaBonus != 0)
+		{
+			mData->pDatas[idx].ButcherNinjaFriendly =
+				AnimatedCounterNew("Ninja bonus: ", ninjaBonus);
+		}
+		const int friendlyBonus = GetFriendlyBonus(pd);
+		if (friendlyBonus != 0)
+		{
+			mData->pDatas[idx].ButcherNinjaFriendly =
+				AnimatedCounterNew("Friendly bonus: ", friendlyBonus);
+		}
+	}
+	idx++;
 	CA_FOREACH_END()
 }
 static GameLoopResult MissionSummaryUpdate(GameLoopData *data, LoopRunner *l)
 {
 	MissionSummaryData *mData = data->Data;
 
-	const GameLoopResult result = MenuUpdate(&mData->ms);
-	if (result == UPDATE_RESULT_OK)
+	GameLoopResult result = MenuUpdate(&mData->ms);
+	if (result == UPDATE_RESULT_DRAW)
 	{
-		gCampaign.IsComplete = mData->completed &&
-			gCampaign.MissionIndex == (int)gCampaign.Setting.Missions.size - 1;
+		bool done = true;
+		done = AnimatedCounterUpdate(&mData->AccessBonus, 1) && done;
+		done = AnimatedCounterUpdate(&mData->TimeBonus, 1) && done;
+		for (int i = 0; i < MAX_LOCAL_PLAYERS; i++)
+		{
+			done = AnimatedCounterUpdate(&mData->pDatas[i].Score, 1) && done;
+			done = AnimatedCounterUpdate(&mData->pDatas[i].Total, 1) && done;
+			done = AnimatedCounterUpdate(&mData->pDatas[i].Time, 1) && done;
+			done =
+				AnimatedCounterUpdate(&mData->pDatas[i].TimeTotal, 1) && done;
+			done = AnimatedCounterUpdate(
+					   &mData->pDatas[i].HealthResurrection, 1) &&
+				   done;
+			done = AnimatedCounterUpdate(
+					   &mData->pDatas[i].ButcherNinjaFriendly, 1) &&
+				   done;
+		}
+
+		// Skip after animations are done if in demo mode
+		if (done && gEventHandlers.DemoQuitTimer > 0)
+		{
+			result = UPDATE_RESULT_OK;
+		}
+	}
+
+	if (result == UPDATE_RESULT_OK || mData->m->missionData->SkipDebrief)
+	{
+		gCampaign.IsComplete =
+			mData->completed &&
+			mData->m->NextMission == (int)gCampaign.Setting.Missions.size;
 		if (gCampaign.IsComplete)
 		{
 			LoopRunnerChange(l, ScreenVictory(&gCampaign));
@@ -555,7 +631,7 @@ static GameLoopResult MissionSummaryUpdate(GameLoopData *data, LoopRunner *l)
 		else if (!mData->completed)
 		{
 			// Check if we want to return to menu or replay mission
-			mData->m->IsQuit = mData->ms.current->u.returnCode == 1;
+			gCampaign.IsQuit = mData->ms.current->u.returnCode == 1;
 			LoopRunnerPop(l);
 		}
 		else
@@ -564,18 +640,6 @@ static GameLoopResult MissionSummaryUpdate(GameLoopData *data, LoopRunner *l)
 				l, HighScoresScreen(&gCampaign, &gGraphicsDevice));
 		}
 	}
-	else
-	{
-		AnimatedCounterUpdate(&mData->AccessBonus, 1);
-		AnimatedCounterUpdate(&mData->TimeBonus, 1);
-		for (int i = 0; i < MAX_LOCAL_PLAYERS; i++)
-		{
-			AnimatedCounterUpdate(&mData->pDatas[i].Score, 1);
-			AnimatedCounterUpdate(&mData->pDatas[i].Total, 1);
-			AnimatedCounterUpdate(&mData->pDatas[i].HealthResurrection, 1);
-			AnimatedCounterUpdate(&mData->pDatas[i].ButcherNinjaFriendly, 1);
-		}
-	}
 	return result;
 }
 static void MissionSummaryDraw(GameLoopData *data)
@@ -587,10 +651,10 @@ static void MissionSummaryDraw(GameLoopData *data)
 static bool AreAnySurvived(void)
 {
 	CA_FOREACH(const PlayerData, p, gPlayerDatas)
-		if (p->survived)
-		{
-			return true;
-		}
+	if (p->survived)
+	{
+		return true;
+	}
 	CA_FOREACH_END()
 	return false;
 }
@@ -600,10 +664,8 @@ static int GetAccessBonus(const struct MissionOptions *m)
 }
 static int GetTimeBonus(const struct MissionOptions *m, int *secondsOut)
 {
-	int seconds =
-		60 +
-		(int)m->missionData->Objectives.size * 30 -
-		m->time / FPS_FRAMELIMIT;
+	int seconds = 60 + (int)m->missionData->Objectives.size * 30 -
+				  m->time / FPS_FRAMELIMIT;
 	if (seconds < 0)
 	{
 		seconds = 0;
@@ -633,8 +695,9 @@ static void ApplyBonuses(PlayerData *p, const int bonus)
 }
 static int GetHealthBonus(const PlayerData *p)
 {
-	const int maxHealth = ModeMaxHealth(gCampaign.Entry.Mode);
-	return p->hp > maxHealth - 50 ? (p->hp + 50 - maxHealth) * 10 : 0;
+	const int maxHealth = p->Char.maxHealth;
+	const int hp = MIN(p->hp, maxHealth);
+	return hp > maxHealth - 50 ? (hp + 50 - p->Char.maxHealth) * 10 : 0;
 }
 static int GetResurrectionFee(const PlayerData *p)
 {
@@ -662,8 +725,8 @@ static int GetNinjaBonus(const PlayerData *p)
 				break;
 			}
 		}
-		if (wc != NULL && !wc->CanShoot && p->Stats.Friendlies == 0 &&
-			p->Stats.Kills > 5)
+		if (wc != NULL && !WeaponClassCanShoot(wc) &&
+			p->Stats.Friendlies == 0 && p->Stats.Kills > 5)
 		{
 			return 50 * p->Stats.Kills;
 		}
@@ -678,8 +741,8 @@ static void DrawPlayerSummary(
 	const struct vec2i pos, const struct vec2i size,
 	const PlayerSummaryDrawData *data);
 static void MissionSummaryMenuDraw(
-	const menu_t *menu, GraphicsDevice *g,
-	const struct vec2i p, const struct vec2i size, const void *data)
+	const menu_t *menu, GraphicsDevice *g, const struct vec2i p,
+	const struct vec2i size, const void *data)
 {
 	UNUSED(menu);
 	UNUSED(p);
@@ -689,74 +752,62 @@ static void MissionSummaryMenuDraw(
 	const int w = gGraphicsDevice.cachedConfig.Res.x;
 	const int h = gGraphicsDevice.cachedConfig.Res.y;
 
-	// Display password
-	if (strlen(gAutosave.LastMission.Password) > 0)
-	{
-		char s[64];
-		sprintf(s, "Last password: %s", gAutosave.LastMission.Password);
-		FontOpts opts = FontOptsNew();
-		opts.HAlign = ALIGN_CENTER;
-		opts.VAlign = ALIGN_END;
-		opts.Area = g->cachedConfig.Res;
-		opts.Pad.y = opts.Area.y / 12;
-		FontStrOpt(s, svec2i_zero(), opts);
-	}
-
 	// Display objectives and bonuses
 	struct vec2i pos = svec2i(w / 6, h / 2 + h / 10);
 	int idx = 1;
 	CA_FOREACH(const Objective, o, mData->m->missionData->Objectives)
-		// Do not mention optional objectives with none completed
-		if (o->done == 0 && !ObjectiveIsRequired(o))
-		{
-			continue;
-		}
+	// Do not mention optional objectives with none completed
+	if (o->done == 0 && !ObjectiveIsRequired(o))
+	{
+		continue;
+	}
 
-		// Objective icon
-		DrawObjectiveInfo(o, svec2i_add(pos, svec2i(-26, FontH())));
+	// Objective icon
+	DrawObjectiveInfo(o, svec2i_add(pos, svec2i(-26, FontH())));
 
-		// Objective completion text
-		char s[100];
-		sprintf(s, "Objective %d: %d of %d, %d required",
-			idx, o->done, o->Count, o->Required);
-		FontOpts opts = FontOptsNew();
-		opts.Area = gGraphicsDevice.cachedConfig.Res;
-		opts.Pad = pos;
-		if (!ObjectiveIsRequired(o))
-		{
-			// Show optional objectives in purple
-			opts.Mask = colorPurple;
-		}
-		FontStrOpt(s, svec2i_zero(), opts);
+	// Objective completion text
+	char s[100];
+	sprintf(
+		s, "Objective %d: %d of %d, %d required", idx, o->done, o->Count,
+		o->Required);
+	FontOpts opts = FontOptsNew();
+	opts.Area = g->cachedConfig.Res;
+	opts.Pad = pos;
+	if (!ObjectiveIsRequired(o))
+	{
+		// Show optional objectives in purple
+		opts.Mask = colorPurple;
+	}
+	FontStrOpt(s, svec2i_zero(), opts);
 
-		// Objective status text
-		opts = FontOptsNew();
-		opts.HAlign = ALIGN_END;
-		opts.Area = gGraphicsDevice.cachedConfig.Res;
-		opts.Pad = pos;
-		if (!ObjectiveIsComplete(o))
-		{
-			opts.Mask = colorRed;
-			FontStrOpt("Failed", svec2i_zero(), opts);
-		}
-		else if (ObjectiveIsPerfect(o) && AreAnySurvived())
-		{
-			opts.Mask = colorGreen;
-			char buf[16];
-			sprintf(buf, "Perfect: %d", PERFECT_BONUS);
-			FontStrOpt(buf, svec2i_zero(), opts);
-		}
-		else if (ObjectiveIsRequired(o))
-		{
-			FontStrOpt("Done", svec2i_zero(), opts);
-		}
-		else
-		{
-			FontStrOpt("Bonus!", svec2i_zero(), opts);
-		}
+	// Objective status text
+	opts = FontOptsNew();
+	opts.HAlign = ALIGN_END;
+	opts.Area = g->cachedConfig.Res;
+	opts.Pad = pos;
+	if (!ObjectiveIsComplete(o))
+	{
+		opts.Mask = colorRed;
+		FontStrOpt("Failed", svec2i_zero(), opts);
+	}
+	else if (ObjectiveIsPerfect(o) && AreAnySurvived())
+	{
+		opts.Mask = colorGreen;
+		char buf[16];
+		sprintf(buf, "Perfect: %d", PERFECT_BONUS);
+		FontStrOpt(buf, svec2i_zero(), opts);
+	}
+	else if (ObjectiveIsRequired(o))
+	{
+		FontStrOpt("Done", svec2i_zero(), opts);
+	}
+	else
+	{
+		FontStrOpt("Bonus!", svec2i_zero(), opts);
+	}
 
-		pos.y += 15;
-		idx++;
+	pos.y += 15;
+	idx++;
 	CA_FOREACH_END()
 
 	// Draw other bonuses
@@ -784,7 +835,7 @@ static void MissionSummaryMenuDraw(
 		DrawPlayerSummary(svec2i_zero(), playerSize, &mData->pDatas[0]);
 		DrawPlayerSummary(svec2i(w / 2, 0), playerSize, &mData->pDatas[1]);
 		break;
-	case 3:	// fallthrough
+	case 3: // fallthrough
 	case 4:
 		// 2x2
 		playerSize = svec2i(w / 2, h / 4);
@@ -811,12 +862,12 @@ static void DrawPlayerSummary(
 	char s[50];
 	const int totalTextHeight = FontH() * 7;
 	// display text on right half
-	struct vec2i textPos = svec2i(
-		pos.x + size.x / 2, CENTER_Y(pos, size, totalTextHeight));
+	struct vec2i textPos =
+		svec2i(pos.x + size.x / 2, CENTER_Y(pos, size, totalTextHeight));
 
 	DisplayCharacterAndName(
-		svec2i_add(pos, svec2i(size.x / 4, size.y / 2)),
-		&data->Pd->Char, DIRECTION_DOWN, data->Pd->name, colorWhite);
+		svec2i_add(pos, svec2i(size.x / 4, size.y / 2)), &data->Pd->Char,
+		DIRECTION_DOWN, data->Pd->name, colorWhite, data->Pd->guns[0]);
 
 	if (data->Pd->survived)
 	{
@@ -832,6 +883,10 @@ static void DrawPlayerSummary(
 	textPos.y += FontH();
 	AnimatedCounterDraw(&data->Total, textPos);
 	textPos.y += FontH();
+	AnimatedCounterTimeDraw(&data->Time, textPos);
+	textPos.y += FontH();
+	AnimatedCounterTimeDraw(&data->TimeTotal, textPos);
+	textPos.y += FontH();
 	sprintf(
 		s, "Missions: %d", data->Pd->missions + (data->Pd->survived ? 1 : 0));
 	FontStr(s, textPos);
@@ -854,37 +909,40 @@ static void DrawObjectiveInfo(const Objective *o, const struct vec2i pos)
 
 	switch (o->Type)
 	{
-	case OBJECTIVE_KILL:
-		{
-			const Character *cd = CArrayGet(
-				&store->OtherChars, CharacterStoreGetSpecialId(store, 0));
-			DrawHead(
-				gGraphicsDevice.gameWindow.renderer, cd, DIRECTION_DOWN, pos);
-		}
-		break;
-	case OBJECTIVE_RESCUE:
-		{
-			const Character *cd = CArrayGet(
-				&store->OtherChars, CharacterStoreGetPrisonerId(store, 0));
-			DrawHead(
-				gGraphicsDevice.gameWindow.renderer, cd, DIRECTION_DOWN, pos);
-		}
-		break;
-	case OBJECTIVE_COLLECT:
+	case OBJECTIVE_KILL: {
+		const Character *cd = CArrayGet(
+			&store->OtherChars, CharacterStoreGetSpecialId(store, 0));
+		DrawHead(gGraphicsDevice.gameWindow.renderer, cd, DIRECTION_DOWN, pos);
+	}
+	break;
+	case OBJECTIVE_RESCUE: {
+		const Character *cd = CArrayGet(
+			&store->OtherChars, CharacterStoreGetPrisonerId(store, 0));
+		DrawHead(gGraphicsDevice.gameWindow.renderer, cd, DIRECTION_DOWN, pos);
+	}
+	break;
+	case OBJECTIVE_COLLECT: {
+		struct vec2i drawPos = pos;
+		CA_FOREACH(const PickupClass *, pc, o->u.Pickups)
 		CPicDraw(
-			&gGraphicsDevice, &o->u.Pickup->Pic,
-			svec2i_subtract(pos, svec2i(-4, -4)), NULL);
-		break;
-	case OBJECTIVE_DESTROY:
-		{
-			struct vec2i picOffset;
-			const Pic *p = MapObjectGetPic(o->u.MapObject, &picOffset);
-			PicRender(
-				p, gGraphicsDevice.gameWindow.renderer,
-				svec2i_add(pos, picOffset), colorWhite, 0, svec2_one(),
-				SDL_FLIP_NONE, Rect2iZero());
-		}
-		break;
+			&gGraphicsDevice, &(*pc)->Pic,
+			svec2i_subtract(drawPos, svec2i(-4, -4)), NULL);
+		drawPos = svec2i_add(drawPos, svec2i(4, 2));
+		CA_FOREACH_END()
+	}
+	break;
+	case OBJECTIVE_DESTROY: {
+		struct vec2i drawPos = pos;
+		CA_FOREACH(const MapObject *, mo, o->u.MapObjects)
+		struct vec2i picOffset;
+		const Pic *p = MapObjectGetPic(*mo, &picOffset);
+		PicRender(
+			p, gGraphicsDevice.gameWindow.renderer, svec2i_add(drawPos, picOffset),
+			colorWhite, 0, svec2_one(), SDL_FLIP_NONE, Rect2iZero());
+		drawPos = svec2i_add(drawPos, svec2i(4, 2));
+		CA_FOREACH_END()
+	}
+	break;
 	case OBJECTIVE_INVESTIGATE:
 		// Don't draw
 		return;
