@@ -22,7 +22,7 @@
 	This file incorporates work covered by the following copyright and
 	permission notice:
 
-	Copyright (c) 2013-2022 Cong Xu
+	Copyright (c) 2013-2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -113,6 +113,8 @@ void ActorSetState(TActor *actor, const ActorAnimation state)
 
 static void ActorUpdateWeapon(TActor *a, Weapon *w, const int ticks);
 static void CheckPickups(TActor *actor);
+static void ActorAddAmmoPickup(const TActor *actor);
+static void ActorAddGunPickup(const TActor *actor);
 void UpdateActorState(TActor *actor, int ticks)
 {
 	ActorUpdateWeapon(actor, ACTOR_GET_GUN(actor), ticks);
@@ -152,8 +154,32 @@ void UpdateActorState(TActor *actor, int ticks)
 		return;
 	}
 
+	const Character *c = ActorGetCharacter(actor);
+
 	if (actor->health <= 0)
 	{
+		if (actor->dead == 0 && !gCampaign.IsClient)
+		{
+			// Just entered dead state
+			// Drop weapon/ammo
+			if (c->Drop)
+			{
+				GameEvent e = GameEventNew(GAME_EVENT_ADD_PICKUP);
+				strcpy(e.u.AddPickup.PickupClass, c->Drop->Name);
+				e.u.AddPickup.Pos = Vec2ToNet(actor->Pos);
+				GameEventsEnqueue(&gGameEvents, e);
+			}
+			else if (!gCampaign.Setting.BuyAndSell)
+			{
+				// Add an ammo pickup of the actor's gun
+				if (gCampaign.Setting.Ammo)
+				{
+					ActorAddAmmoPickup(actor);
+				}
+
+				ActorAddGunPickup(actor);
+			}
+		}
 		actor->dead++;
 		actor->MoveVel = svec2_zero();
 		actor->stateCounter = 4;
@@ -180,7 +206,7 @@ void UpdateActorState(TActor *actor, int ticks)
 	{
 		actor->DrawRadians -= (float)MIN(DRAW_RADIAN_SPEED * ticks, dr);
 	}
-	
+
 	const struct vec2i tilePos = Vec2ToTile(actor->Pos);
 	const Tile *t = MapGetTile(&gMap, tilePos);
 
@@ -188,17 +214,18 @@ void UpdateActorState(TActor *actor, int ticks)
 	// Step on 2 and 6
 	// TODO: custom animation and footstep frames
 	const int frame = AnimationGetFrame(&actor->anim);
-	const bool isFootstepFrame = actor->anim.Type == ACTORANIMATION_WALKING && (frame == 2 || frame == 6) && actor->anim.newFrame;
+	const bool isFootstepFrame = actor->anim.Type == ACTORANIMATION_WALKING &&
+								 (frame == 2 || frame == 6) &&
+								 actor->anim.newFrame;
 	if (isFootstepFrame)
 	{
 
 		if (ConfigGetBool(&gConfig, "Sound.Footsteps"))
 		{
 			GameEvent e = GameEventNew(GAME_EVENT_SOUND_AT);
-			const CharacterClass *cc = ActorGetCharacter(actor)->Class;
-			MatGetFootstepSound(cc, t, e.u.SoundAt.Sound);
+			MatGetFootstepSound(c->Class, t, e.u.SoundAt.Sound);
 			e.u.SoundAt.Pos = Vec2ToNet(actor->thing.Pos);
-			e.u.SoundAt.Distance = cc->FootstepsDistancePlus;
+			e.u.SoundAt.Distance = c->Class->FootstepsDistancePlus;
 			GameEventsEnqueue(&gGameEvents, e);
 		}
 
@@ -233,18 +260,19 @@ void UpdateActorState(TActor *actor, int ticks)
 	}
 
 	// Damage when on special tiles
-	if (!gCampaign.IsClient &&
-		actor->anim.Type == ACTORANIMATION_WALKING ? isFootstepFrame : (gMission.time % FPS_FRAMELIMIT) == 0)
+	if (!gCampaign.IsClient && actor->anim.Type == ACTORANIMATION_WALKING
+			? isFootstepFrame
+			: (gMission.time % FPS_FRAMELIMIT) == 0)
 	{
-	   const BulletClass *b = MatGetDamageBullet(t);
-	   if (b != NULL)
-	   {
-		   GameEvent e = GameEventNew(GAME_EVENT_THING_DAMAGE);
-		   e.u.ThingDamage.UID = actor->uid;
-		   e.u.ThingDamage.Kind = KIND_CHARACTER;
-		   BulletToDamageEvent(b, &e);
-		   GameEventsEnqueue(&gGameEvents, e);
-	   }
+		const BulletClass *b = MatGetDamageBullet(t);
+		if (b != NULL)
+		{
+			GameEvent e = GameEventNew(GAME_EVENT_THING_DAMAGE);
+			e.u.ThingDamage.UID = actor->uid;
+			e.u.ThingDamage.Kind = KIND_CHARACTER;
+			BulletToDamageEvent(b, &e);
+			GameEventsEnqueue(&gGameEvents, e);
+		}
 	}
 
 	// Animation
@@ -292,21 +320,23 @@ static void ActorUpdateWeapon(TActor *a, Weapon *w, const int ticks)
 	}
 }
 
+static bool CanMeleeTarget(
+	const TActor *a, const Weapon *w, const Thing *target);
 static struct vec2 GetConstrainedPos(
 	const Map *map, const struct vec2 from, const struct vec2 to,
 	const struct vec2i size);
 static void OnMove(TActor *a);
-bool TryMoveActor(TActor *actor, struct vec2 pos)
+static bool TryMoveActor(TActor *actor, struct vec2 pos)
 {
 	CASSERT(
 		!svec2_is_nearly_equal(actor->Pos, pos, EPSILON_POS),
 		"trying to move to same position");
+	actor->CanPickupSpecial = false;
 
 	// Don't check collisions for pilots
 	if (actor->vehicleUID == -1)
 	{
 		actor->hasCollided = true;
-		actor->CanPickupSpecial = false;
 
 		const struct vec2 oldPos = actor->Pos;
 		pos = GetConstrainedPos(&gMap, actor->Pos, pos, actor->thing.size);
@@ -319,68 +349,79 @@ bool TryMoveActor(TActor *actor, struct vec2 pos)
 		const CollisionParams params = {
 			THING_IMPASSABLE, CalcCollisionTeam(true, actor),
 			IsPVP(gCampaign.Entry.Mode), false};
-		Thing *target = OverlapGetFirstItem(
+		const Thing *target = OverlapGetFirstItem(
 			&actor->thing, pos, actor->thing.size, actor->thing.Vel, params);
-		if (target)
+		if (target && actor->health > 0)
 		{
-			Weapon *gun = ACTOR_GET_WEAPON(actor);
-			const TObject *object = target->kind == KIND_OBJECT
-										? CArrayGet(&gObjs, target->id)
-										: NULL;
-			const int barrel = ActorGetCanFireBarrel(actor, gun);
-			// Check for melee damage if we are the owner of the actor
-			const bool checkMelee =
-				(!gCampaign.IsClient && actor->PlayerUID < 0) ||
-				ActorIsLocalPlayer(actor->uid);
-			// TODO: support melee weapons on multi guns
-			const BulletClass *b = WeaponClassGetBullet(gun->Gun, barrel);
-			if (checkMelee && barrel >= 0 && !WeaponClassCanShoot(gun->Gun) &&
-				actor->health > 0 && b &&
-				(!object ||
-				 (((b->Hit.Object.Hit && target->kind == KIND_OBJECT) ||
-				   (b->Hit.Flesh.Hit && target->kind == KIND_CHARACTER)) &&
-				  !ObjIsDangerous(object))))
+			// We are running into an object or actor
+			// If we have an auto-melee weapon and aren't shooting, switch to
+			// it
+			const Weapon *meleeW = &actor->guns[MELEE_SLOT];
+			if (actor->gunIndex != MELEE_SLOT && meleeW->Gun &&
+				meleeW->Gun->Type == GUNTYPE_MELEE &&
+				meleeW->Gun->u.Normal.Auto && !actor->hasShot &&
+				CanMeleeTarget(actor, meleeW, target))
 			{
-				if (CanHit(b, actor->flags, actor->uid, target))
+				GameEvent e = GameEventNew(GAME_EVENT_ACTOR_SWITCH_GUN);
+				e.u.ActorSwitchGun.UID = actor->uid;
+				e.u.ActorSwitchGun.GunIdx = MELEE_SLOT;
+				GameEventsEnqueue(&gGameEvents, e);
+			}
+			else
+			{
+				Weapon *gun = ACTOR_GET_WEAPON(actor);
+				if (CanMeleeTarget(actor, gun, target))
 				{
-					// Tell the server that we want to melee something
-					GameEvent e = GameEventNew(GAME_EVENT_ACTOR_MELEE);
-					e.u.Melee.UID = actor->uid;
-					strcpy(e.u.Melee.BulletClass, b->Name);
-					e.u.Melee.TargetKind = target->kind;
-					switch (target->kind)
+					const int barrel = ActorGetCanFireBarrel(actor, gun);
+					if (barrel >= 0)
 					{
-					case KIND_CHARACTER:
-						e.u.Melee.TargetUID =
-							((const TActor *)CArrayGet(&gActors, target->id))
-								->uid;
-						e.u.Melee.HitType = HIT_FLESH;
-						break;
-					case KIND_OBJECT:
-						e.u.Melee.TargetUID =
-							((const TObject *)CArrayGet(&gObjs, target->id))
-								->uid;
-						e.u.Melee.HitType = HIT_OBJECT;
-						break;
-					default:
-						CASSERT(false, "cannot damage target kind");
-						break;
-					}
-					if (gun->barrels[barrel].soundLock > 0)
-					{
-						e.u.Melee.HitType = (int)HIT_NONE;
-					}
-					GameEventsEnqueue(&gGameEvents, e);
-					WeaponBarrelOnFire(gun, barrel);
+						// TODO: support melee weapons on multi guns
+						const BulletClass *b =
+							WeaponClassGetBullet(gun->Gun, barrel);
+						if (CanHit(b, actor->flags, actor->uid, target))
+						{
+							// Tell the server that we want to melee something
+							GameEvent e = GameEventNew(GAME_EVENT_ACTOR_MELEE);
+							e.u.Melee.UID = actor->uid;
+							strcpy(e.u.Melee.BulletClass, b->Name);
+							e.u.Melee.TargetKind = target->kind;
+							switch (target->kind)
+							{
+							case KIND_CHARACTER:
+								e.u.Melee.TargetUID =
+									((const TActor *)CArrayGet(
+										 &gActors, target->id))
+										->uid;
+								e.u.Melee.HitType = HIT_FLESH;
+								break;
+							case KIND_OBJECT:
+								e.u.Melee.TargetUID =
+									((const TObject *)CArrayGet(
+										 &gObjs, target->id))
+										->uid;
+								e.u.Melee.HitType = HIT_OBJECT;
+								break;
+							default:
+								CASSERT(false, "cannot damage target kind");
+								break;
+							}
+							if (gun->barrels[barrel].soundLock > 0)
+							{
+								e.u.Melee.HitType = (int)HIT_NONE;
+							}
+							GameEventsEnqueue(&gGameEvents, e);
+							WeaponBarrelOnFire(gun, barrel);
 
-					// Only set grimace when counter 0 so that the actor
-					// alternates their grimace
-					if (actor->grimaceCounter == 0)
-					{
-						actor->grimaceCounter = GRIMACE_MELEE_TICKS;
+							// Only set grimace when counter 0 so that the
+							// actor alternates their grimace
+							if (actor->grimaceCounter == 0)
+							{
+								actor->grimaceCounter = GRIMACE_MELEE_TICKS;
+							}
+						}
 					}
+					return false;
 				}
-				return false;
 			}
 
 			const struct vec2 yPos = svec2(actor->Pos.x, pos.y);
@@ -418,6 +459,37 @@ bool TryMoveActor(TActor *actor, struct vec2 pos)
 	actor->hasCollided = false;
 	return true;
 }
+static bool CanMeleeTarget(
+	const TActor *a, const Weapon *w, const Thing *target)
+{
+	// Check valid melee weapon
+	if (!w->Gun || WeaponClassCanShoot(w->Gun))
+		return false;
+	// Check for melee damage if we are the owner of the actor
+	// Is client: must be local player
+	// Not client: AI or local player
+	if (!ActorIsLocalPlayer(a->uid) &&
+		(gCampaign.IsClient || a->PlayerUID >= 0))
+		return false;
+	// Check if any barrel can fire and melee
+	for (int barrel = 0; barrel < WeaponClassNumBarrels(w->Gun); barrel++)
+	{
+		const BulletClass *b = WeaponClassGetBullet(w->Gun, barrel);
+		if (b == NULL)
+			continue;
+		if (target->kind == KIND_OBJECT)
+		{
+			if (!b->Hit.Object.Hit)
+				continue;
+			const TObject *object = CArrayGet(&gObjs, target->id);
+			if (object && !ObjIsDangerous(object))
+				return true;
+		}
+		else if (b->Hit.Flesh.Hit && target->kind == KIND_CHARACTER)
+			return true;
+	}
+	return false;
+}
 // Get a movement position that is constrained by collisions
 // May return a position that is the same as the 'from', that is, we cannot
 // move in the direction specified.
@@ -657,15 +729,16 @@ static void CheckRescue(const TActor *a)
 	}
 }
 
-void ActorHeal(TActor *actor, int health)
+void ActorHeal(TActor *actor, const int amount, const bool exceedMax)
 {
-	actor->health += health;
-	actor->health = MIN(actor->health, ActorGetCharacter(actor)->maxHealth);
+	actor->health += amount;
+	const int maxHealth = ActorGetMaxHeal(actor, exceedMax);
+	actor->health = MIN(actor->health, maxHealth);
 	AddParticle ap;
 	memset(&ap, 0, sizeof ap);
 	ap.Pos = actor->Pos;
 	ap.Z = 10;
-	for (int i = 0; i < MAX(health / 20, 1); i++)
+	for (int i = 0; i < MAX(amount / 20, 1); i++)
 	{
 		ap.Vel = svec2(RAND_FLOAT(-0.2f, 0.2f), RAND_FLOAT(-0.2f, 0.2f));
 		EmitterStart(&actor->healEffect, &ap);
@@ -739,6 +812,12 @@ bool ActorUsesAmmo(const TActor *actor, const int ammoId)
 	return false;
 }
 
+static void ActorSetChatter(TActor *a, const char *text, const int count)
+{
+	strcpy(a->Chatter, text);
+	a->ChatterCounter = count;
+}
+
 void ActorReplaceGun(const NActorReplaceGun rg)
 {
 	TActor *a = ActorGetByUID(rg.UID);
@@ -756,17 +835,16 @@ void ActorReplaceGun(const NActorReplaceGun rg)
 	Weapon w = WeaponCreate(wc);
 	memcpy(&a->guns[rg.GunIdx], &w, sizeof w);
 	// Switch immediately to picked up gun
-	const PlayerData *p = PlayerDataGetByUID(a->PlayerUID);
-	if (wc->Type == GUNTYPE_GRENADE && PlayerHasGrenadeButton(p))
-	{
-		a->grenadeIndex = rg.GunIdx - MAX_GUNS;
-	}
-	else
+	if (ActorCanSwitchToWeapon(a, rg.GunIdx))
 	{
 		a->gunIndex = rg.GunIdx;
 	}
-
 	SoundPlayAt(&gSoundDevice, wc->SwitchSound, a->Pos);
+	// Show gun switch text as chatter, but only for PCs
+	if (a->PlayerUID >= 0)
+	{
+		ActorSetChatter(a, wc->name, CHATTER_SWITCH_GUN);
+	}
 }
 
 int ActorFindGun(const TActor *a, const WeaponClass *wc)
@@ -818,12 +896,6 @@ int ActorGetNumGrenades(const TActor *a)
 	return count;
 }
 
-static void ActorSetChatter(TActor *a, const char *text, const int count)
-{
-	strcpy(a->Chatter, text);
-	a->ChatterCounter = count;
-}
-
 // Set AI state and possibly say something based on the state
 void ActorSetAIState(TActor *actor, const AIState s)
 {
@@ -869,6 +941,17 @@ static void FireWeapon(TActor *a, Weapon *w)
 	{
 		return;
 	}
+	// If currently equipping an auto-melee weapon,
+	// and if possible, switch to the last gun
+	if (w->Gun->Type == GUNTYPE_MELEE && w->Gun->u.Normal.Auto &&
+		a->guns[a->lastGunIdx].Gun != NULL)
+	{
+		GameEvent e = GameEventNew(GAME_EVENT_ACTOR_SWITCH_GUN);
+		e.u.ActorSwitchGun.UID = a->uid;
+		e.u.ActorSwitchGun.GunIdx = a->lastGunIdx;
+		GameEventsEnqueue(&gGameEvents, e);
+		return;
+	}
 	const int barrel = ActorGetCanFireBarrel(a, w);
 	if (barrel == -1)
 	{
@@ -918,10 +1001,10 @@ static bool ActorTryChangeDirection(
 {
 	const bool willChangeDirecton =
 		!actor->petrified && CMD_HAS_DIRECTION(cmd) &&
-		(!(cmd & CMD_BUTTON2) ||
+		(!Button2(cmd) ||
 		 ConfigGetEnum(&gConfig, "Game.SwitchMoveStyle") !=
 			 SWITCHMOVE_STRAFE) &&
-		(!(prevCmd & CMD_BUTTON1) ||
+		(!Button1(prevCmd) ||
 		 ConfigGetEnum(&gConfig, "Game.FireMoveStyle") != FIREMOVE_STRAFE);
 	const direction_e dir = CmdToDirection(cmd);
 	if (willChangeDirecton && dir != actor->direction)
@@ -938,7 +1021,7 @@ static bool ActorTryChangeDirection(
 
 static bool ActorTryShoot(TActor *actor, const int cmd)
 {
-	const bool willShoot = !actor->petrified && (cmd & CMD_BUTTON1);
+	const bool willShoot = !actor->petrified && Button1(cmd);
 	if (willShoot)
 	{
 		FireWeapon(actor, ACTOR_GET_GUN(actor));
@@ -972,9 +1055,10 @@ static bool TryGrenade(TActor *a, const int cmd)
 	return willGrenade;
 }
 
-static bool ActorTryMove(TActor *actor, int cmd, int hasShot, int ticks);
+static bool ActorTryMove(TActor *actor, int cmd, int ticks);
 void CommandActor(TActor *actor, int cmd, int ticks)
 {
+	actor->hasShot = false;
 	// If this is a pilot, command the vehicle instead
 	if (actor->vehicleUID != -1)
 	{
@@ -997,12 +1081,13 @@ void CommandActor(TActor *actor, int cmd, int ticks)
 		{
 			const bool hasChangedDirection =
 				ActorTryChangeDirection(actor, cmd, actor->lastCmd);
-			const bool hasShot = ActorTryShoot(actor, cmd);
+			actor->hasShot = ActorTryShoot(actor, cmd);
 			const bool hasGrenaded = TryGrenade(actor, cmd);
-			const bool hasMoved = ActorTryMove(actor, cmd, hasShot, ticks);
+			const bool hasMoved = ActorTryMove(actor, cmd, ticks);
 			ActorAnimation anim = actor->anim.Type;
 			// Idle if player hasn't done anything
-			if (!(hasChangedDirection || hasShot || hasGrenaded || hasMoved))
+			if (!(hasChangedDirection || actor->hasShot || hasGrenaded ||
+				  hasMoved))
 			{
 				anim = ACTORANIMATION_IDLE;
 			}
@@ -1025,10 +1110,10 @@ void CommandActor(TActor *actor, int cmd, int ticks)
 	}
 
 	actor->specialCmdDir = CMD_HAS_DIRECTION(cmd);
-	if ((cmd & CMD_BUTTON2) && !actor->specialCmdDir)
+	if (Button2(cmd) && !actor->specialCmdDir)
 	{
 		// Special: pick up things that can only be picked up on demand
-		if (!actor->PickupAll && !(actor->lastCmd & CMD_BUTTON2) &&
+		if (!actor->PickupAll && !Button2(actor->lastCmd) &&
 			actor->vehicleUID == -1)
 		{
 			GameEvent e = GameEventNew(GAME_EVENT_ACTOR_PICKUP_ALL);
@@ -1040,28 +1125,31 @@ void CommandActor(TActor *actor, int cmd, int ticks)
 
 	actor->lastCmd = cmd;
 }
-static bool ActorTryMove(TActor *actor, int cmd, int hasShot, int ticks)
+static bool ActorTryMove(TActor *actor, int cmd, int ticks)
 {
 	const bool canMoveWhenShooting =
-		ConfigGetEnum(&gConfig, "Game.FireMoveStyle") != FIREMOVE_STOP ||
-		!hasShot ||
+	actor->PlayerUID < 0 ? (actor->flags & FLAGS_MOVE_AND_SHOOT) :
+	(
+	 ConfigGetEnum(&gConfig, "Game.FireMoveStyle") != FIREMOVE_STOP ||
 		(ConfigGetEnum(&gConfig, "Game.SwitchMoveStyle") ==
 			 SWITCHMOVE_STRAFE &&
-		 (cmd & CMD_BUTTON2));
+		 Button2(cmd))
+	 );
+	const bool canMove = !actor->hasShot || canMoveWhenShooting;
 	const bool willMove =
-		!actor->petrified && CMD_HAS_DIRECTION(cmd) && canMoveWhenShooting;
+		!actor->petrified && CMD_HAS_DIRECTION(cmd) && canMove;
 	actor->MoveVel = svec2_zero();
 	if (willMove)
 	{
 		const float moveAmount = ActorGetCharacter(actor)->speed * ticks;
 		struct vec2 moveVel = svec2_zero();
-		if (cmd & CMD_LEFT)
+		if (Left(cmd))
 			moveVel.x--;
-		else if (cmd & CMD_RIGHT)
+		else if (Right(cmd))
 			moveVel.x++;
-		if (cmd & CMD_UP)
+		if (Up(cmd))
 			moveVel.y--;
-		else if (cmd & CMD_DOWN)
+		else if (Down(cmd))
 			moveVel.y++;
 		if (!svec2_is_zero(moveVel))
 		{
@@ -1101,13 +1189,13 @@ void SlideActor(TActor *actor, int cmd)
 	GameEvent e = GameEventNew(GAME_EVENT_ACTOR_SLIDE);
 	e.u.ActorSlide.UID = actor->uid;
 	struct vec2 vel = svec2_zero();
-	if (cmd & CMD_LEFT)
+	if (Left(cmd))
 		vel.x = -SLIDE_X;
-	else if (cmd & CMD_RIGHT)
+	else if (Right(cmd))
 		vel.x = SLIDE_X;
-	if (cmd & CMD_UP)
+	if (Up(cmd))
 		vel.y = -SLIDE_Y;
-	else if (cmd & CMD_DOWN)
+	else if (Down(cmd))
 		vel.y = SLIDE_Y;
 	e.u.ActorSlide.Vel = Vec2ToNet(vel);
 	GameEventsEnqueue(&gGameEvents, e);
@@ -1367,36 +1455,12 @@ static void CheckManualPilot(TActor *a, const CollisionParams params)
 	// TODO: co-op AI pilot
 	a->CanPickupSpecial = true;
 }
-static void ActorAddAmmoPickup(const TActor *actor);
-static void ActorAddGunPickup(const TActor *actor);
 static void ActorDie(TActor *actor)
 {
-	const Character *c = ActorGetCharacter(actor);
-	GameEvent e;
-	if (!gCampaign.IsClient)
-	{
-		if (c->Drop)
-		{
-			e = GameEventNew(GAME_EVENT_ADD_PICKUP);
-			strcpy(e.u.AddPickup.PickupClass, c->Drop->Name);
-			e.u.AddPickup.Pos = Vec2ToNet(actor->Pos);
-			GameEventsEnqueue(&gGameEvents, e);
-		}
-		else
-		{
-			// Add an ammo pickup of the actor's gun
-			if (gCampaign.Setting.Ammo)
-			{
-				ActorAddAmmoPickup(actor);
-			}
-
-			ActorAddGunPickup(actor);
-		}
-	}
-
 	// Add corpse
 	if (ConfigGetEnum(&gConfig, "Graphics.Gore") != GORE_NONE)
 	{
+		const Character *c = ActorGetCharacter(actor);
 		GameEvent ea = GameEventNew(GAME_EVENT_MAP_OBJECT_ADD);
 		ea.u.MapObjectAdd.UID = ObjsGetNextUID();
 		const MapObject *corpse = StrMapObject(c->Class->Corpse);
@@ -1412,9 +1476,12 @@ static void ActorDie(TActor *actor)
 		GameEventsEnqueue(&gGameEvents, ea);
 	}
 
-	e = GameEventNew(GAME_EVENT_ACTOR_DIE);
+	GameEvent e = GameEventNew(GAME_EVENT_ACTOR_DIE);
 	e.u.ActorDie.UID = actor->uid;
 	GameEventsEnqueue(&gGameEvents, e);
+	
+	// Persist player ammo for when they respawn
+	ActorPersistPlayerWeaponsAndAmmo(actor);
 }
 static bool IsUnarmedBot(const TActor *actor);
 static void ActorAddAmmoPickup(const TActor *actor)
@@ -1628,14 +1695,16 @@ TActor *ActorAdd(NActorAdd aa)
 			AmmoGetById(&gAmmo, i)->Amount * AMMO_STARTING_MULTIPLE;
 		CArrayPushBack(&actor->ammo, &amount);
 	}
-	if (gMission.missionData->WeaponPersist)
+	if (gMission.missionData && gMission.missionData->WeaponPersist)
 	{
 		for (int i = 0; i < aa.Ammo_count; i++)
 		{
-			// Use persisted ammo amount if it is greater
-			if ((int)aa.Ammo[i].Amount >
-				AmmoGetById(&gAmmo, aa.Ammo[i].Id)->Amount *
-					AMMO_STARTING_MULTIPLE)
+			// Use persisted ammo amount if it is greater,
+			// or if buy/sell is enabled
+			if (gCampaign.Setting.BuyAndSell ||
+				(int)aa.Ammo[i].Amount >
+					AmmoGetById(&gAmmo, aa.Ammo[i].Id)->Amount *
+						AMMO_STARTING_MULTIPLE)
 			{
 				CArraySet(&actor->ammo, aa.Ammo[i].Id, &aa.Ammo[i].Amount);
 			}
@@ -1656,10 +1725,6 @@ TActor *ActorAdd(NActorAdd aa)
 			{
 				actor->gunIndex = i;
 			}
-			if (i >= MAX_GUNS && ACTOR_GET_GRENADE(actor)->Gun == NULL)
-			{
-				actor->grenadeIndex = i - MAX_GUNS;
-			}
 		}
 		p->ActorUID = aa.UID;
 	}
@@ -1669,6 +1734,11 @@ TActor *ActorAdd(NActorAdd aa)
 		Weapon gun = WeaponCreate(c->Gun);
 		actor->guns[0] = gun;
 		actor->gunIndex = 0;
+		if (c->Melee)
+		{
+			Weapon melee = WeaponCreate(c->Melee);
+			actor->guns[MELEE_SLOT] = melee;
+		}
 	}
 	actor->health = aa.Health;
 	actor->action = ACTORACTION_MOVING;
@@ -1717,12 +1787,22 @@ TActor *ActorAdd(NActorAdd aa)
 		ActorSetAIState(actor, AI_STATE_IDLE);
 	}
 
-	EmitterInit(
-		&actor->barrelSmoke, StrParticleClass(&gParticleClasses, "smoke"),
-		svec2_zero(), -0.05f, 0.05f, 3, 3, 0, 0, 10);
-	EmitterInit(
-		&actor->healEffect, StrParticleClass(&gParticleClasses, "health_plus"),
-		svec2_zero(), -0.1f, 0.1f, 0, 0, 0, 0, 0);
+	const ParticleClass *barrelSmoke =
+		StrParticleClass(&gParticleClasses, "smoke");
+	if (barrelSmoke)
+	{
+		EmitterInit(
+			&actor->barrelSmoke, barrelSmoke, svec2_zero(), -0.05f, 0.05f, 3,
+			3, 0, 0, 10);
+	}
+	const ParticleClass *healthPlus =
+		StrParticleClass(&gParticleClasses, "health_plus");
+	if (healthPlus)
+	{
+		EmitterInit(
+			&actor->healEffect, healthPlus, svec2_zero(), -0.1f, 0.1f, 0, 0, 0,
+			0, 0);
+	}
 	GoreEmitterInit(&actor->blood1, "blood1");
 	GoreEmitterInit(&actor->blood2, "blood2");
 	GoreEmitterInit(&actor->blood3, "blood3");
@@ -1818,19 +1898,53 @@ int ActorGetCanFireBarrel(const TActor *a, const Weapon *w)
 	}
 	return barrel;
 }
-bool ActorTrySwitchWeapon(const TActor *a, const bool allGuns)
+bool ActorCanSwitchToWeapon(const TActor *a, const int slot)
+{
+	const WeaponClass *wc = a->guns[slot].Gun;
+	if (wc == NULL)
+	{
+		return false;
+	}
+	// Don't cycle to auto-melee or default melee unless there are no guns with
+	// ammo
+	if (slot == MELEE_SLOT && wc->u.Normal.Auto)
+	{
+		for (int i = 0; i < MELEE_SLOT; i++)
+		{
+			const WeaponClass *wc2 = a->guns[i].Gun;
+			if (wc2 != NULL)
+			{
+				if (!gCampaign.Setting.Ammo)
+				{
+					return false;
+				}
+				for (int j = 0; j < WeaponClassNumBarrels(wc2); j++)
+				{
+					if (ActorWeaponGetAmmo(a, wc2, j) != 0)
+					{
+						return false;
+					}
+				}
+			}
+		}
+	}
+	// Don't switch to grenade, unless the player has no grenade button
+	if (slot == GRENADE_SLOT)
+	{
+		const PlayerData *p = PlayerDataGetByUID(a->PlayerUID);
+		return !PlayerHasGrenadeButton(p);
+	}
+	return true;
+}
+bool ActorTrySwitchWeapon(const TActor *a)
 {
 	// Find the next weapon to switch to
-	// If the player does not have a grenade key set, allow switching to
-	// grenades (classic style)
-	const int switchCount = allGuns ? MAX_WEAPONS : MAX_GUNS;
-	const int startIndex =
-		ActorGetNumGuns(a) > 0 ? a->gunIndex : a->grenadeIndex + MAX_GUNS;
+	const int startIndex = ActorGetNumGuns(a) > 0 ? a->gunIndex : MAX_GUNS;
 	int weaponIndex = startIndex;
 	do
 	{
-		weaponIndex = (weaponIndex + 1) % switchCount;
-	} while (a->guns[weaponIndex].Gun == NULL);
+		weaponIndex = (weaponIndex + 1) % MAX_WEAPONS;
+	} while (!ActorCanSwitchToWeapon(a, weaponIndex));
 	if (weaponIndex == startIndex)
 	{
 		// No other weapon to switch to
@@ -1849,9 +1963,92 @@ void ActorSwitchGun(const NActorSwitchGun sg)
 	if (a == NULL || !a->isInUse)
 		return;
 	a->gunIndex = sg.GunIdx;
+	if (sg.GunIdx < MELEE_SLOT)
+	{
+		a->lastGunIdx = sg.GunIdx;
+	}
 	const WeaponClass *gun = ACTOR_GET_WEAPON(a)->Gun;
 	SoundPlayAt(&gSoundDevice, gun->SwitchSound, a->thing.Pos);
-	ActorSetChatter(a, gun->name, CHATTER_SWITCH_GUN);
+	// Show gun switch text as chatter, but only for PCs
+	if (a->PlayerUID >= 0)
+	{
+		ActorSetChatter(a, gun->name, CHATTER_SWITCH_GUN);
+	}
+}
+
+void ActorPickupGun(const TActor *a, const WeaponClass *wc)
+{
+	int actorsGunIdx = ActorFindGun(a, wc);
+	if (actorsGunIdx >= 0)
+	{
+		// Actor already has gun
+
+		// Switch to the same gun
+		if (ActorCanSwitchToWeapon(a, actorsGunIdx))
+		{
+			GameEvent e = GameEventNew(GAME_EVENT_ACTOR_SWITCH_GUN);
+			e.u.ActorSwitchGun.UID = a->uid;
+			e.u.ActorSwitchGun.GunIdx = actorsGunIdx;
+			GameEventsEnqueue(&gGameEvents, e);
+		}
+
+		// Drop the same gun
+		PickupAddGun(wc, a->Pos);
+	}
+	else
+	{
+		// Make sure weapons are picked up in the correct slot
+		int slotStart, slotEnd;
+		GunTypeGetSlotStartEnd(wc->Type, &slotStart, &slotEnd);
+		if (actorsGunIdx < slotStart || actorsGunIdx > slotEnd)
+		{
+			actorsGunIdx = slotStart;
+		}
+
+		// Pickup gun
+		// Replace the current gun, unless there's a free slot, in which case
+		// pick up into the free spot
+		GameEvent e = GameEventNew(GAME_EVENT_ACTOR_REPLACE_GUN);
+		e.u.ActorReplaceGun.UID = a->uid;
+		strcpy(e.u.ActorReplaceGun.Gun, wc->name);
+		switch (wc->Type)
+		{
+		case GUNTYPE_MELEE:
+			e.u.ActorReplaceGun.GunIdx = MELEE_SLOT;
+			break;
+		case GUNTYPE_GRENADE:
+			e.u.ActorReplaceGun.GunIdx = GRENADE_SLOT;
+			break;
+		default:
+			if (a->gunIndex < MELEE_SLOT)
+			{
+				e.u.ActorReplaceGun.GunIdx = a->gunIndex;
+			}
+			else
+			{
+				e.u.ActorReplaceGun.GunIdx = 0;
+			}
+			break;
+		}
+		int replaceGunIndex = e.u.ActorReplaceGun.GunIdx;
+		for (int i = slotStart; i <= slotEnd; i++)
+		{
+			if (a->guns[i].Gun == NULL)
+			{
+				e.u.ActorReplaceGun.GunIdx = i;
+				replaceGunIndex = -1;
+				break;
+			}
+		}
+		GameEventsEnqueue(&gGameEvents, e);
+
+		// If replacing a gun, "drop" the gun being replaced (i.e. create a gun
+		// pickup)
+		if (replaceGunIndex >= 0)
+		{
+			PickupAddGun(a->guns[replaceGunIndex].Gun, a->Pos);
+		}
+	}
 }
 
 bool ActorIsImmune(const TActor *actor, const special_damage_e damage)
@@ -2098,6 +2295,12 @@ static void ActorAddBloodSplatters(
 	}
 }
 
+int ActorGetMaxHeal(const TActor *a, const bool exceedMax)
+{
+	const Character *c = ActorGetCharacter(a);
+	return exceedMax ? c->excessHealth : c->maxHealth;
+}
+
 int ActorGetHealthPercent(const TActor *a)
 {
 	const int maxHealth = ActorGetCharacter(a)->maxHealth;
@@ -2138,3 +2341,18 @@ bool ActorIsLocalPlayer(const int uid)
 
 	return PlayerIsLocal(a->PlayerUID);
 }
+
+void ActorPersistPlayerWeaponsAndAmmo(const TActor *a)
+{
+	// Update the player's weapons and ammo based on their actor
+	PlayerData *p = PlayerDataGetByUID(a->PlayerUID);
+	if (p == NULL)
+	{
+		return;
+	}
+	for (int i = 0; i < MAX_WEAPONS; i++)
+	{
+		p->guns[i] = a->guns[i].Gun;
+	}
+	CArrayCopy(&p->ammo, &a->ammo);
+}
