@@ -579,7 +579,6 @@ Config ConfigDefault(void)
 	ConfigGroupAdd(&game,
 		ConfigNewInt("Lives", 2, 1, 5, 1, NULL, NULL));
 	ConfigGroupAdd(&game, ConfigNewBool("HealthPickups", true));
-	ConfigGroupAdd(&game, ConfigNewBool("Ammo", false));
 	ConfigGroupAdd(&game, ConfigNewBool("Fog", true));
 	ConfigGroupAdd(&game,
 		ConfigNewInt("SightRange", 15, 8, 40, 1, NULL, NULL));
@@ -614,8 +613,6 @@ Config ConfigDefault(void)
 		"Brightness", 0, BLIT_BRIGHTNESS_MIN, BLIT_BRIGHTNESS_MAX, 1,
 		NULL, NULL));
 
-	// Full screen setting not needed for macOS
-#ifndef __APPLE__
 	ConfigGroupAdd(&gfx, ConfigNewBool("Fullscreen",
 #ifdef __GCWZERO__
 		true
@@ -623,7 +620,6 @@ Config ConfigDefault(void)
 		false
 #endif
 		));
-#endif
 
 	ConfigGroupAdd(&gfx,
 		ConfigNewInt("WindowWidth", 640, 320, 4096, 0, NULL, NULL));
@@ -699,7 +695,6 @@ Config ConfigDefault(void)
 	ConfigGroupAdd(&snd,
 		ConfigNewInt("SoundVolume", 64, 0, 64, 8, NULL, Div8Str));
 	ConfigGroupAdd(&snd, ConfigNewBool("Footsteps", true));
-	ConfigGroupAdd(&snd, ConfigNewBool("Hits", true));
 	ConfigGroupAdd(&snd, ConfigNewBool("Reloads", true));
 	ConfigGroupAdd(&root, snd);
 
