@@ -1,30 +1,30 @@
 /*
-    C-Dogs SDL
-    A port of the legendary (and fun) action/arcade cdogs.
+	C-Dogs SDL
+	A port of the legendary (and fun) action/arcade cdogs.
 
-    Copyright (c) 2014-2017, Cong Xu
-    All rights reserved.
+	Copyright (c) 2014-2017, 2021, 2023 Cong Xu
+	All rights reserved.
 
-    Redistribution and use in source and binary forms, with or without
-    modification, are permitted provided that the following conditions are met:
+	Redistribution and use in source and binary forms, with or without
+	modification, are permitted provided that the following conditions are met:
 
-    Redistributions of source code must retain the above copyright notice, this
-    list of conditions and the following disclaimer.
-    Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
+	Redistributions of source code must retain the above copyright notice, this
+	list of conditions and the following disclaimer.
+	Redistributions in binary form must reproduce the above copyright notice,
+	this list of conditions and the following disclaimer in the documentation
+	and/or other materials provided with the distribution.
 
-    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-    ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-    POSSIBILITY OF SUCH DAMAGE.
+	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+	POSSIBILITY OF SUCH DAMAGE.
 */
 #include "net_server.h"
 
@@ -48,7 +48,6 @@
 
 NetServer gNetServer;
 
-
 void NetServerInit(NetServer *n)
 {
 	memset(n, 0, sizeof *n);
@@ -70,7 +69,7 @@ void NetServerOpen(NetServer *n)
 	{
 		return;
 	}
-	
+
 	n->server = HostOpen();
 	if (n->server == NULL)
 	{
@@ -86,7 +85,7 @@ void NetServerOpen(NetServer *n)
 	// Save limited length hostname for LAN scanners
 	// This is because getting hostname locally is much faster
 	if (enet_address_get_host(
-		&n->server->address, n->hostname, sizeof n->hostname) != 0)
+			&n->server->address, n->hostname, sizeof n->hostname) != 0)
 	{
 		LOG(LM_NET, LL_WARN, "Failed to get hostname");
 		n->hostname[0] = '\0';
@@ -97,7 +96,8 @@ static ENetHost *HostOpen(void)
 	ENetAddress address;
 	address.host = ENET_HOST_ANY;
 	address.port = ENET_PORT_ANY;
-	ENetHost *host = enet_host_create(&address, NET_SERVER_MAX_CLIENTS, 2, 0, 0);
+	ENetHost *host =
+		enet_host_create(&address, NET_SERVER_MAX_CLIENTS, 2, 0, 0);
 	if (host == NULL)
 	{
 		LOG(LM_NET, LL_ERROR, "cannot create server host");
@@ -107,8 +107,8 @@ static ENetHost *HostOpen(void)
 	{
 		char buf[256];
 		enet_address_get_host_ip(&host->address, buf, sizeof buf);
-		LOG(LM_NET, LL_INFO, "starting server on %s:%u",
-			buf, host->address.port);
+		LOG(LM_NET, LL_INFO, "starting server on %s:%u", buf,
+			host->address.port);
 	}
 	return host;
 }
@@ -187,15 +187,14 @@ void NetServerPoll(NetServer *n)
 		{
 			switch (event.type)
 			{
-			case ENET_EVENT_TYPE_CONNECT:
-				{
-					char buf[256];
-					enet_address_get_host_ip(
-						&event.peer->address, buf, sizeof buf);
-					LOG(LM_NET, LL_INFO, "client connection from %s:%u",
-						buf, event.peer->address.port);
-				}
-				break;
+			case ENET_EVENT_TYPE_CONNECT: {
+				char buf[256];
+				enet_address_get_host_ip(
+					&event.peer->address, buf, sizeof buf);
+				LOG(LM_NET, LL_INFO, "client connection from %s:%u", buf,
+					event.peer->address.port);
+			}
+			break;
 			case ENET_EVENT_TYPE_RECEIVE:
 				OnReceive(n, event);
 				break;
@@ -242,13 +241,13 @@ static void PollListener(NetServer *n)
 	strcpy(sinfo.Hostname, n->hostname);
 	sinfo.GameMode = gCampaign.Entry.Mode;
 	sinfo.CampaignName[0] = '\0';
-	strncat(sinfo.CampaignName, gCampaign.Setting.Title,
+	strncat(
+		sinfo.CampaignName, gCampaign.Setting.Title,
 		sizeof sinfo.CampaignName - 1);
 	sinfo.MissionNumber = gCampaign.MissionIndex + 1;
 	sinfo.NumPlayers = GetNumPlayers(PLAYER_ANY, false, false);
-	sinfo.MaxPlayers =
-		NET_SERVER_MAX_CLIENTS * MAX_LOCAL_PLAYERS +
-		GetNumPlayers(PLAYER_ANY, false, true);
+	sinfo.MaxPlayers = NET_SERVER_MAX_CLIENTS * MAX_LOCAL_PLAYERS +
+					   GetNumPlayers(PLAYER_ANY, false, true);
 	// Encode our packet
 	uint8_t encbuf[1024];
 	pb_ostream_t stream = pb_ostream_from_buffer(encbuf, sizeof encbuf);
@@ -256,7 +255,8 @@ static void PollListener(NetServer *n)
 	CASSERT(status, "Failed to encode pb");
 	recvbuf.data = encbuf;
 	recvbuf.dataLength = stream.bytes_written;
-	if (enet_socket_send(n->listen, &addr, &recvbuf, 1) != (int)recvbuf.dataLength)
+	if (enet_socket_send(n->listen, &addr, &recvbuf, 1) !=
+		(int)recvbuf.dataLength)
 	{
 		LOG(LM_NET, LL_ERROR, "Failed to reply to scanner");
 	}
@@ -264,14 +264,14 @@ static void PollListener(NetServer *n)
 static void OnConnect(NetServer *n, ENetEvent event);
 static void OnReceive(NetServer *n, ENetEvent event)
 {
-	const GameEventType msg = (GameEventType)*(uint32_t *)event.packet->data;
+	const GameEventType msg = (GameEventType) * (uint32_t *)event.packet->data;
 	int peerId = -1;
 	if (event.peer->data != NULL)
 	{
 		// We may not have assigned peer ID
 		peerId = ((NetPeerData *)event.peer->data)->Id;
-		LOG(LM_NET, LL_TRACE, "recv message from peerId(%d) msg(%d)",
-			peerId, (int)msg);
+		LOG(LM_NET, LL_TRACE, "recv message from peerId(%d) msg(%d)", peerId,
+			(int)msg);
 	}
 	const GameEventEntry gee = GameEventGetEntry(msg);
 	if (gee.Enqueue)
@@ -292,19 +292,20 @@ static void OnReceive(NetServer *n, ENetEvent event)
 		case GAME_EVENT_CLIENT_READY:
 			CASSERT(peerId >= 0, "peer id unset");
 			// Flush game events to make sure we add the players
-			HandleGameEvents(&gGameEvents, NULL, NULL, NULL);
+			HandleGameEvents(&gGameEvents, NULL, NULL, NULL, NULL);
 			// Reset player data
 			for (int i = 0; i < MAX_LOCAL_PLAYERS; i++)
 			{
 				const int cid = (peerId + 1) * MAX_LOCAL_PLAYERS + i;
 				const PlayerData *pData = PlayerDataGetByUID(cid);
-				if (pData == NULL) continue;
+				if (pData == NULL)
+					continue;
 				GameEvent e = GameEventNew(GAME_EVENT_PLAYER_DATA);
 				e.u.PlayerData = PlayerDataMissionReset(pData);
 				GameEventsEnqueue(&gGameEvents, e);
 			}
 			// Flush game events to make sure we reset player data
-			HandleGameEvents(&gGameEvents, NULL, NULL, NULL);
+			HandleGameEvents(&gGameEvents, NULL, NULL, NULL, NULL);
 
 			// Send game start messages if we've started already
 			if (gMission.HasStarted)
@@ -333,9 +334,10 @@ static void OnReceive(NetServer *n, ENetEvent event)
 			}
 			if (gMission.HasBegun)
 			{
-				NGameBegin gb = NGameBegin_init_default;
-				gb.MissionTime = gMission.time;
-				NetServerSendMsg(n, peerId, GAME_EVENT_GAME_BEGIN, &gb);
+				GameEvent e = GameEventNew(GAME_EVENT_GAME_BEGIN);
+				e.u.GameBegin.MissionTime = gMission.time;
+				NetServerSendMsg(
+					n, peerId, GAME_EVENT_GAME_BEGIN, &e.u.GameBegin);
 			}
 
 			NetServerFlush(n);
@@ -351,8 +353,8 @@ static void OnConnect(NetServer *n, ENetEvent event)
 {
 	char buf[256];
 	enet_address_get_host_ip(&event.peer->address, buf, sizeof buf);
-	LOG(LM_NET, LL_INFO, "new client connected from %s:%u",
-		buf, event.peer->address.port);
+	LOG(LM_NET, LL_INFO, "new client connected from %s:%u", buf,
+		event.peer->address.port);
 	/* Store any relevant client information here. */
 	CMALLOC(event.peer->data, sizeof(NetPeerData));
 	const int peerId = n->peerId;
@@ -371,7 +373,7 @@ static void OnConnect(NetServer *n, ENetEvent event)
 	NCampaignDef def = NMakeCampaignDef(&gCampaign);
 	NetServerSendMsg(n, peerId, GAME_EVENT_CAMPAIGN_DEF, &def);
 
-	SoundPlay(&gSoundDevice, StrSound("hahaha"));
+	SoundPlay(&gSoundDevice, StrSound("menu_start"));
 	LOG(LM_NET, LL_DEBUG, "NetServer: client connection complete");
 
 	NetServerFlush(n);
@@ -388,8 +390,8 @@ static void OnDisconnect(const ENetEvent event)
 	CASSERT(peerId >= 0, "Cannot find disconnected peer id");
 	char buf[256];
 	enet_address_get_host_ip(&event.peer->address, buf, sizeof buf);
-	LOG(LM_NET, LL_INFO, "peerId(%d) disconnected %s:%u",
-		peerId, buf, event.peer->address.port);
+	LOG(LM_NET, LL_INFO, "peerId(%d) disconnected %s:%u", peerId, buf,
+		event.peer->address.port);
 	// Remove client's players
 	for (int i = 0; i < MAX_LOCAL_PLAYERS; i++)
 	{
@@ -401,7 +403,8 @@ static void OnDisconnect(const ENetEvent event)
 
 void NetServerFlush(NetServer *n)
 {
-	if (n->server == NULL) return;
+	if (n->server == NULL)
+		return;
 	enet_host_flush(n->server);
 }
 
@@ -409,18 +412,20 @@ static void SendConfig(
 	Config *config, const char *name, NetServer *n, const int peerId);
 void NetServerSendGameStartMessages(NetServer *n, const int peerId)
 {
+	if (!n->server)
+		return;
+	GameEvent e;
 	// Send details of all current players
 	CA_FOREACH(const PlayerData, pOther, gPlayerDatas)
-		NPlayerData pd = NMakePlayerData(pOther);
-		NetServerSendMsg(n, peerId, GAME_EVENT_PLAYER_DATA, &pd);
-		LOG(LM_NET, LL_DEBUG, "send player data uid(%d) maxHealth(%d)",
-			(int)pd.UID, (int)pd.MaxHealth);
+	NPlayerData pd = NMakePlayerData(pOther);
+	NetServerSendMsg(n, peerId, GAME_EVENT_PLAYER_DATA, &pd);
+	LOG(LM_NET, LL_DEBUG, "send player data uid(%d) maxHealth(%d)  excessHealth(%d) HP(%d)",
+		(int)pd.UID, (int)pd.MaxHealth, (int)pd.ExcessHealth, (int)pd.HP);
 	CA_FOREACH_END()
 
 	// Send all game-specific config values
 	SendConfig(&gConfig, "Game.FriendlyFire", n, peerId);
 	SendConfig(&gConfig, "Game.FPS", n, peerId);
-	SendConfig(&gConfig, "Game.Ammo", n, peerId);
 	SendConfig(&gConfig, "Game.Fog", n, peerId);
 	SendConfig(&gConfig, "Game.SightRange", n, peerId);
 	SendConfig(&gConfig, "Game.AllyCollision", n, peerId);
@@ -429,39 +434,42 @@ void NetServerSendGameStartMessages(NetServer *n, const int peerId)
 
 	// Send all actors
 	CA_FOREACH(const TActor, a, gActors)
-		if (!a->isInUse)
-		{
-			continue;
-		}
-		NActorAdd aa = NActorAdd_init_default;
-		aa.UID = a->uid;
-		aa.CharId = a->charId;
-		aa.Health = a->health;
-		aa.Direction = (int32_t)a->direction;
-		aa.PlayerUID = a->PlayerUID;
-		aa.ThingFlags = a->thing.flags;
-		aa.Pos = Vec2ToNet(a->Pos);
-		LOG(LM_NET, LL_DEBUG, "send add actor UID(%d) playerUID(%d)",
-			(int)aa.UID, (int)aa.PlayerUID);
-		NetServerSendMsg(n, peerId, GAME_EVENT_ACTOR_ADD, &aa);
+	if (!a->isInUse)
+	{
+		continue;
+	}
+	e = GameEventNew(GAME_EVENT_ACTOR_ADD);
+	e.u.ActorAdd.UID = a->uid;
+	e.u.ActorAdd.PilotUID = a->pilotUID;
+	e.u.ActorAdd.VehicleUID = a->vehicleUID;
+	e.u.ActorAdd.CharId = a->charId;
+	e.u.ActorAdd.Health = a->health;
+	e.u.ActorAdd.Direction = (int32_t)a->direction;
+	e.u.ActorAdd.PlayerUID = a->PlayerUID;
+	e.u.ActorAdd.ThingFlags = a->thing.flags;
+	e.u.ActorAdd.Pos = Vec2ToNet(a->Pos);
+	LOG(LM_NET, LL_DEBUG, "send add actor UID(%d) playerUID(%d)",
+		(int)e.u.ActorAdd.UID, (int)e.u.ActorAdd.PlayerUID);
+	NetServerSendMsg(n, peerId, GAME_EVENT_ACTOR_ADD, &e.u.ActorAdd);
 	CA_FOREACH_END()
 
 	// Send key state
-	NAddKeys ak = NAddKeys_init_default;
-	ak.KeyFlags = gMission.KeyFlags;
-	NetServerSendMsg(n, peerId, GAME_EVENT_ADD_KEYS, &ak);
+	e = GameEventNew(GAME_EVENT_ADD_KEYS);
+	e.u.AddKeys.KeyFlags = gMission.KeyFlags;
+	NetServerSendMsg(n, peerId, GAME_EVENT_ADD_KEYS, &e.u.AddKeys);
 
 	// Send objective counts
 	CA_FOREACH(const Objective, o, gMission.missionData->Objectives)
-		NObjectiveUpdate ou = NObjectiveUpdate_init_default;
-		ou.ObjectiveId = _ca_index;
-		ou.Count = o->done;
-		NetServerSendMsg(n, peerId, GAME_EVENT_OBJECTIVE_UPDATE, &ou);
+	e = GameEventNew(GAME_EVENT_OBJECTIVE_UPDATE);
+	e.u.ObjectiveUpdate.ObjectiveId = _ca_index;
+	e.u.ObjectiveUpdate.Count = o->done;
+	NetServerSendMsg(
+		n, peerId, GAME_EVENT_OBJECTIVE_UPDATE, &e.u.ObjectiveUpdate);
 	CA_FOREACH_END()
 
 	// Send all tiles, RLE
 	const Tile *tLast = NULL;
-	NTileSet ts = NTileSet_init_default;
+	e = GameEventNew(GAME_EVENT_TILE_SET);
 	struct vec2i pos;
 	for (pos.y = 0; pos.y < gMap.Size.y; pos.y++)
 	{
@@ -471,110 +479,132 @@ void NetServerSendGameStartMessages(NetServer *n, const int peerId)
 			// Use RLE, so check if the current tile is the same as the last
 			if (tLast != NULL && t->Class == tLast->Class)
 			{
-				ts.RunLength++;
+				e.u.TileSet.RunLength++;
 			}
 			else
 			{
 				// Send the last run
 				if (tLast != NULL)
 				{
-					NetServerSendMsg(n, peerId, GAME_EVENT_TILE_SET, &ts);
+					NetServerSendMsg(
+						n, peerId, GAME_EVENT_TILE_SET, &e.u.TileSet);
 				}
 				// Begin the next run
-				memset(&ts, 0, sizeof ts);
-				ts.Pos = Vec2i2Net(pos);
-				if (t->Class != NULL && t->Class->Name)
+				e = GameEventNew(GAME_EVENT_TILE_SET);
+				e.u.TileSet.Pos = Vec2i2Net(pos);
+				if (t->Class != NULL)
 				{
-					strcpy(ts.ClassName, t->Class->Name);
+					TileClassGetName(
+						e.u.TileSet.ClassName, t->Class, t->Class->Style,
+						t->Class->StyleType, t->Class->Mask,
+						t->Class->MaskAlt);
 				}
-				ts.RunLength = 0;
+				if (t->Door.Class != NULL)
+				{
+					TileClassGetName(
+						e.u.TileSet.DoorClassName, t->Door.Class,
+						t->Door.Class->Style, t->Door.Class->StyleType,
+						t->Door.Class->Mask, t->Door.Class->MaskAlt);
+				}
+				if (t->Door.Class2 != NULL)
+				{
+					TileClassGetName(
+						e.u.TileSet.DoorClass2Name, t->Door.Class2,
+						t->Door.Class2->Style, t->Door.Class2->StyleType,
+						t->Door.Class2->Mask, t->Door.Class2->MaskAlt);
+				}
+				e.u.TileSet.RunLength = 0;
 			}
 			tLast = t;
 		}
 	}
-	NetServerSendMsg(n, peerId, GAME_EVENT_TILE_SET, &ts);
+	NetServerSendMsg(n, peerId, GAME_EVENT_TILE_SET, &e.u.TileSet);
 
 	// Send all the tiles visited so far
-	NExploreTiles et = NExploreTiles_init_default;
-	et.Runs_count = 0;
-	et.Runs[0].Run = 0;
+	e = GameEventNew(GAME_EVENT_EXPLORE_TILES);
 	bool run = false;
 	for (pos.y = 0; pos.y < gMap.Size.y; pos.y++)
 	{
 		for (pos.x = 0; pos.x < gMap.Size.x; pos.x++)
 		{
 			const Tile *t = MapGetTile(&gMap, pos);
-			if (LOSAddRun(&et, &run, pos, t->isVisited))
+			if (LOSAddRun(&e.u.ExploreTiles, &run, pos, t->isVisited))
 			{
-				NetServerSendMsg(n, peerId, GAME_EVENT_EXPLORE_TILES, &et);
-				et.Runs_count = 0;
-				et.Runs[0].Run = 0;
+				NetServerSendMsg(
+					n, peerId, GAME_EVENT_EXPLORE_TILES, &e.u.ExploreTiles);
+				e = GameEventNew(GAME_EVENT_EXPLORE_TILES);
 				run = false;
 			}
 		}
 	}
-	if (et.Runs_count > 0)
+	if (e.u.ExploreTiles.Runs_count > 0)
 	{
-		NetServerSendMsg(n, peerId, GAME_EVENT_EXPLORE_TILES, &et);
+		NetServerSendMsg(
+			n, peerId, GAME_EVENT_EXPLORE_TILES, &e.u.ExploreTiles);
 	}
 
 	// Send all pickups
 	CA_FOREACH(const Pickup, p, gPickups)
-		if (!p->isInUse) continue;
-		NAddPickup api = NAddPickup_init_default;
-		api.UID = p->UID;
-		strcpy(api.PickupClass, p->class->Name);
-		api.IsRandomSpawned = p->IsRandomSpawned;
-		api.SpawnerUID = p->SpawnerUID;
-		api.ThingFlags = p->thing.flags;
-		api.Pos = Vec2ToNet(p->thing.Pos);
-		NetServerSendMsg(n, peerId, GAME_EVENT_ADD_PICKUP, &api);
+	if (!p->isInUse)
+		continue;
+	e = GameEventNew(GAME_EVENT_ADD_PICKUP);
+	e.u.AddPickup.UID = p->UID;
+	strcpy(e.u.AddPickup.PickupClass, p->class->Name);
+	e.u.AddPickup.IsRandomSpawned = p->IsRandomSpawned;
+	e.u.AddPickup.SpawnerUID = p->SpawnerUID;
+	e.u.AddPickup.ThingFlags = p->thing.flags;
+	e.u.AddPickup.Pos = Vec2ToNet(p->thing.Pos);
+	NetServerSendMsg(n, peerId, GAME_EVENT_ADD_PICKUP, &e.u.AddPickup);
 	CA_FOREACH_END()
 
 	// Send all map objects
 	CA_FOREACH(const TObject, o, gObjs)
-		if (!o->isInUse) continue;
-		NMapObjectAdd amo = NMapObjectAdd_init_default;
-		amo.UID = o->uid;
-		strcpy(amo.MapObjectClass, o->Class->Name);
-		amo.Pos = Vec2ToNet(o->thing.Pos);
-		amo.ThingFlags = o->thing.flags;
-		amo.Health = o->Health;
-		LOG(LM_NET, LL_DEBUG,
-			"send add map object UID(%d) pos(%d, %d) flags(%x) health(%d)",
-			(int)amo.UID, amo.Pos.x, amo.Pos.y, amo.ThingFlags, amo.Health);
-		NetServerSendMsg(n, peerId, GAME_EVENT_MAP_OBJECT_ADD, &amo);
+	if (!o->isInUse)
+		continue;
+	e = GameEventNew(GAME_EVENT_MAP_OBJECT_ADD);
+	e.u.MapObjectAdd.UID = o->uid;
+	strcpy(e.u.MapObjectAdd.MapObjectClass, o->Class->Name);
+	e.u.MapObjectAdd.Mask = Color2Net(o->thing.CPic.Mask);
+	e.u.MapObjectAdd.Pos = Vec2ToNet(o->thing.Pos);
+	e.u.MapObjectAdd.ThingFlags = o->thing.flags;
+	e.u.MapObjectAdd.Health = o->Health;
+	LOG(LM_NET, LL_DEBUG,
+		"send add map object UID(%d) pos(%d, %d) flags(%x) health(%d)",
+		(int)e.u.MapObjectAdd.UID, e.u.MapObjectAdd.Pos.x,
+		e.u.MapObjectAdd.Pos.y, e.u.MapObjectAdd.ThingFlags,
+		e.u.MapObjectAdd.Health);
+	NetServerSendMsg(n, peerId, GAME_EVENT_MAP_OBJECT_ADD, &e.u.MapObjectAdd);
 	CA_FOREACH_END()
 
 	// If mission complete already, send message
 	if (CanCompleteMission(&gMission))
 	{
-		NMissionComplete mc = NMakeMissionComplete(&gMission, &gMap);
+		NMissionComplete mc = NMakeMissionComplete(&gMission);
 		NetServerSendMsg(n, peerId, GAME_EVENT_MISSION_COMPLETE, &mc);
 	}
 }
 static void SendConfig(
 	Config *config, const char *name, NetServer *n, const int peerId)
 {
-	NConfig msg = NConfig_init_default;
+	GameEvent e = GameEventNew(GAME_EVENT_CONFIG);
 	const Config *c = ConfigGet(config, name);
-	strcpy(msg.Name, name);
+	strcpy(e.u.Config.Name, name);
 	switch (c->Type)
 	{
 	case CONFIG_TYPE_STRING:
 		CASSERT(false, "unimplemented");
 		break;
 	case CONFIG_TYPE_INT:
-		sprintf(msg.Value, "%d", c->u.Int.Value);
+		sprintf(e.u.Config.Value, "%d", c->u.Int.Value);
 		break;
 	case CONFIG_TYPE_FLOAT:
-		sprintf(msg.Value, "%f", c->u.Float.Value);
+		sprintf(e.u.Config.Value, "%f", c->u.Float.Value);
 		break;
 	case CONFIG_TYPE_BOOL:
-		strcpy(msg.Value, c->u.Bool.Value ? "true" : "false");
+		strcpy(e.u.Config.Value, c->u.Bool.Value ? "true" : "false");
 		break;
 	case CONFIG_TYPE_ENUM:
-		sprintf(msg.Value, "%d", (int)c->u.Enum.Value);
+		sprintf(e.u.Config.Value, "%d", (int)c->u.Enum.Value);
 		break;
 	case CONFIG_TYPE_GROUP:
 		CASSERT(false, "Cannot send groups over net");
@@ -583,18 +613,19 @@ static void SendConfig(
 		CASSERT(false, "Unknown config type");
 		break;
 	}
-	NetServerSendMsg(n, peerId, GAME_EVENT_CONFIG, &msg);
+	NetServerSendMsg(n, peerId, GAME_EVENT_CONFIG, &e.u.Config);
 }
 
 void NetServerSendMsg(
 	NetServer *n, const int peerId, const GameEventType e, const void *data)
 {
-	if (!n->server) return;
+	if (!n->server)
+		return;
 
 	if (peerId >= 0)
 	{
-		LOG(LM_NET, LL_TRACE, "send msg(%d) to peers(%d)",
-			(int)e, (int)n->server->connectedPeers);
+		LOG(LM_NET, LL_TRACE, "send msg(%d) to peers(%d)", (int)e,
+			(int)n->server->connectedPeers);
 		// Find the peer and send
 		for (int i = 0; i < (int)n->server->peerCount; i++)
 		{
@@ -610,8 +641,8 @@ void NetServerSendMsg(
 	}
 	else
 	{
-		LOG(LM_NET, LL_TRACE, "bcast msg(%d) to peers(%d)",
-			(int)e, (int)n->server->connectedPeers);
+		LOG(LM_NET, LL_TRACE, "bcast msg(%d) to peers(%d)", (int)e,
+			(int)n->server->connectedPeers);
 		enet_host_broadcast(n->server, 0, NetEncode(e, data));
 	}
 }
