@@ -1,7 +1,7 @@
 /*
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
-	Copyright (c) 2014-2015, 2017-2020, 2022 Cong Xu
+	Copyright (c) 2014-2015, 2017-2020, 2022-2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -133,7 +133,8 @@ void PickupsUpdate(CArray *pickups, const int ticks)
 static bool TreatAsGunPickup(const PickupEffect *pe, const TActor *a);
 static bool TryPickupAmmo(TActor *a, const Pickup *p, const PickupEffect *pe);
 static bool TryPickupGun(
-	TActor *a, const PickupEffect *pe, const bool pickupAll, const char **sound);
+	TActor *a, const PickupEffect *pe, const bool pickupAll,
+	const char **sound);
 void PickupPickup(TActor *a, Pickup *p, const bool pickupAll)
 {
 	if (p->PickedUp)
@@ -177,13 +178,14 @@ void PickupPickup(TActor *a, Pickup *p, const bool pickupAll)
 
 	case PICKUP_HEALTH:
 		// Don't pick up unless taken damage
-		if (a->health < ActorGetCharacter(a)->maxHealth)
+		if (a->health < ActorGetMaxHeal(a, pe->u.Heal.ExceedMax))
 		{
 			canPickup = true;
 			GameEvent e = GameEventNew(GAME_EVENT_ACTOR_HEAL);
 			e.u.Heal.UID = a->uid;
 			e.u.Heal.PlayerUID = a->PlayerUID;
-			e.u.Heal.Amount = pe->u.Health;
+			e.u.Heal.Amount = pe->u.Heal.Amount;
+			e.u.Heal.ExceedMax = pe->u.Heal.ExceedMax;
 			e.u.Heal.IsRandomSpawned = p->IsRandomSpawned;
 			GameEventsEnqueue(&gGameEvents, e);
 		}
@@ -222,7 +224,7 @@ void PickupPickup(TActor *a, Pickup *p, const bool pickupAll)
 		GameEventsEnqueue(&gGameEvents, e);
 	}
 	break;
-			
+
 	case PICKUP_LIVES: {
 		canPickup = true;
 		GameEvent e = GameEventNew(GAME_EVENT_PLAYER_ADD_LIVES);
@@ -254,7 +256,6 @@ void PickupPickup(TActor *a, Pickup *p, const bool pickupAll)
 		// Prevent multiple pickups by marking
 		p->PickedUp = true;
 		a->PickupAll = false;
-		a->CanPickupSpecial = false;
 	}
 }
 
@@ -277,10 +278,8 @@ static bool TreatAsGunPickup(const PickupEffect *pe, const TActor *a)
 		return false;
 	case PICKUP_GUN: {
 		const WeaponClass *wc = IdWeaponClass(pe->u.GunId);
-		// TODO: support picking up multi guns?
-		return wc->Type == GUNTYPE_NORMAL ||
-			   (wc->Type == GUNTYPE_GRENADE &&
-				!HasGunUsingAmmo(a, wc->u.Normal.AmmoId));
+		return wc->Type != GUNTYPE_GRENADE ||
+			   !HasGunUsingAmmo(a, wc->u.Normal.AmmoId);
 	}
 	default:
 		CASSERT(false, "unexpected pickup type");
@@ -335,7 +334,8 @@ static bool TryPickupAmmo(TActor *a, const Pickup *p, const PickupEffect *pe)
 	return true;
 }
 static bool TryPickupGun(
-	TActor *a, const PickupEffect *pe, const bool pickupAll, const char **sound)
+	TActor *a, const PickupEffect *pe, const bool pickupAll,
+	const char **sound)
 {
 	// Guns can only be picked up manually
 	if (!pickupAll)
@@ -354,57 +354,9 @@ static bool TryPickupGun(
 	const WeaponClass *wc =
 		pe->Type == PICKUP_GUN
 			? IdWeaponClass(pe->u.GunId)
-			: StrWeaponClass(
-				  AmmoGetById(&gAmmo, pe->u.Ammo.Id)->DefaultGun);
-	const int actorsGunIdx = ActorFindGun(a, wc);
+			: StrWeaponClass(AmmoGetById(&gAmmo, pe->u.Ammo.Id)->DefaultGun);
 
-	if (actorsGunIdx >= 0)
-	{
-		// Actor already has gun
-
-		// Switch to the same gun
-		GameEvent e = GameEventNew(GAME_EVENT_ACTOR_SWITCH_GUN);
-		e.u.ActorSwitchGun.UID = a->uid;
-		e.u.ActorSwitchGun.GunIdx = actorsGunIdx;
-		GameEventsEnqueue(&gGameEvents, e);
-
-		// Drop the same gun
-		PickupAddGun(wc, a->Pos);
-	}
-	else
-	{
-		// Pickup gun
-		// Replace the current gun, unless there's a free slot, in which case
-		// pick up into the free spot
-		const int weaponIndexStart =
-			wc->Type == GUNTYPE_GRENADE ? MAX_GUNS : 0;
-		const int weaponIndexEnd =
-			wc->Type == GUNTYPE_GRENADE ? MAX_WEAPONS : MAX_GUNS;
-		GameEvent e = GameEventNew(GAME_EVENT_ACTOR_REPLACE_GUN);
-		e.u.ActorReplaceGun.UID = a->uid;
-		strcpy(e.u.ActorReplaceGun.Gun, wc->name);
-		e.u.ActorReplaceGun.GunIdx = wc->Type == GUNTYPE_GRENADE
-										 ? a->grenadeIndex + MAX_GUNS
-										 : a->gunIndex;
-		int replaceGunIndex = e.u.ActorReplaceGun.GunIdx;
-		for (int i = weaponIndexStart; i < weaponIndexEnd; i++)
-		{
-			if (a->guns[i].Gun == NULL)
-			{
-				e.u.ActorReplaceGun.GunIdx = i;
-				replaceGunIndex = -1;
-				break;
-			}
-		}
-		GameEventsEnqueue(&gGameEvents, e);
-
-		// If replacing a gun, "drop" the gun being replaced (i.e. create a gun
-		// pickup)
-		if (replaceGunIndex >= 0)
-		{
-			PickupAddGun(a->guns[replaceGunIndex].Gun, a->Pos);
-		}
-	}
+	ActorPickupGun(a, wc);
 
 	// If the player has less ammo than the default amount,
 	// replenish up to this amount
