@@ -22,7 +22,7 @@
 	This file incorporates work covered by the following copyright and
 	permission notice:
 
-	Copyright (c) 2013-2017, 2019-2021 Cong Xu
+	Copyright (c) 2013-2017, 2019-2022 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -130,6 +130,7 @@ void PathGetBasenameWithoutExtension(char *buf, const char *path)
 	PathGetWithoutExtension(buf, basename);
 }
 
+static void TrimSlashes(char *s);
 #ifdef _WIN32
 #include "sys_config.h"
 #define realpath(src, dst) _fullpath(dst, src, CDOGS_PATH_MAX)
@@ -233,18 +234,12 @@ void RealPath(const char *src, char *dest)
 		{
 			*c = '/';
 		}
-#ifdef _WIN32
-		else
-		{
-			*c = (char)toupper(*c);
-		}
-#endif
 	}
+	TrimSlashes(dest);
 }
 
 // Convert an absolute path to a relative path
 // e.g. /a/path/from/here, /a/path/to -> ../to
-static void TrimSlashes(char *s);
 void RelPath(char *buf, const char *to, const char *from)
 {
 #ifdef _WIN32
@@ -605,10 +600,26 @@ bool IntsEqual(const void *v1, const void *v2)
 	return *(const int *)v1 == *(const int *)v2;
 }
 
+const char *HeadPartStr(const HeadPart hp)
+{
+	switch (hp)
+	{
+		T2S(HEAD_PART_HAIR, "Hair");
+		T2S(HEAD_PART_FACEHAIR, "Facial Hair");
+		T2S(HEAD_PART_HAT, "Hat");
+		T2S(HEAD_PART_GLASSES, "Glasses");
+	default:
+		return "";
+	}
+}
+
 BodyPart StrBodyPart(const char *s)
 {
 	S2T(BODY_PART_HEAD, "head");
 	S2T(BODY_PART_HAIR, "hair");
+	S2T(BODY_PART_FACEHAIR, "facehair");
+	S2T(BODY_PART_HAT, "hat");
+	S2T(BODY_PART_GLASSES, "glasses");
 	S2T(BODY_PART_BODY, "body");
 	S2T(BODY_PART_LEGS, "legs");
 	S2T(BODY_PART_GUN_R, "gun_r");
