@@ -1,7 +1,7 @@
 /*
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
-	Copyright (c) 2017-2021 Cong Xu
+	Copyright (c) 2017-2021, 2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -42,14 +42,16 @@ typedef struct
 	CampaignSetting *Setting;
 	bool *FileChanged;
 	char *CharacterClassNames;
-	char *HairNames;
+	char *HeadPartNames[HEAD_PART_COUNT];
 	char *GunNames;
+	char *MeleeNames;
 	char *PickupNames;
 	CArray texidsChars; // of GLuint[BODY_PART_COUNT]
 	GLuint texidsPreview[BODY_PART_COUNT];
 	CArray texIdsCharClasses; // of GLuint
-	CArray texIdsHairs;		  // of GLuint
+	CArray texIdsHeadParts[HEAD_PART_COUNT];		  // of GLuint
 	CArray texIdsGuns;		  // of GLuint
+	CArray texIdsMelees;		  // of GLuint
 	CArray texIdsPickups;	  // of GLuint
 	Animation anim;
 	direction_e previewDir;
@@ -58,13 +60,32 @@ typedef struct
 } EditorContext;
 
 static const char *IndexCharacterClassName(const int i);
-static const char *IndexHairName(const int i);
+static const char *IndexHeadPartName(const int i, const HeadPart hp);
+static const char *IndexHairName(const int i)
+{
+	return IndexHeadPartName(i, HEAD_PART_HAIR);
+}
+static const char *IndexFacehairName(const int i)
+{
+	return IndexHeadPartName(i, HEAD_PART_FACEHAIR);
+}
+static const char *IndexHatName(const int i)
+{
+	return IndexHeadPartName(i, HEAD_PART_HAT);
+}
+static const char *IndexGlassesName(const int i)
+{
+	return IndexHeadPartName(i, HEAD_PART_GLASSES);
+}
 static const char *IndexGunName(const int i);
+static const char *IndexMeleeName(const int i);
 static const char *IndexPickupName(const int i);
 static const WeaponClass *IndexWeaponClassReal(const int i);
+static const WeaponClass *IndexWeaponClassMelee(const int i);
 static const PickupClass *IndexPickupClass(const int i);
 static int NumCharacterClasses(void);
 static int NumGuns(void);
+static int NumMelees(void);
 static int NumPickups(void);
 static void AddCharacterTextures(EditorContext *ec);
 static bool Draw(SDL_Window *win, struct nk_context *ctx, void *data);
@@ -93,9 +114,16 @@ void CharEditor(
 	ec.FileChanged = fileChanged;
 	ec.CharacterClassNames =
 		GetClassNames(NumCharacterClasses(), IndexCharacterClassName);
-	ec.HairNames =
-		GetClassNames(gPicManager.hairstyleNames.size, IndexHairName);
+	const char *(*indexHeadPartFuncs[HEAD_PART_COUNT])(int) = {
+		IndexHairName, IndexFacehairName, IndexHatName, IndexGlassesName
+	};
+	for (HeadPart hp = HEAD_PART_HAIR; hp < HEAD_PART_COUNT; hp++)
+	{
+		ec.HeadPartNames[hp] =
+		 GetClassNames(gPicManager.headPartNames[hp].size, *indexHeadPartFuncs[hp]);
+	}
 	ec.GunNames = GetClassNames(NumGuns(), IndexGunName);
+	ec.MeleeNames = GetClassNames(NumMelees(), IndexMeleeName);
 	ec.PickupNames = GetClassNames(NumPickups(), IndexPickupName);
 
 	CArrayInit(&ec.texidsChars, sizeof(GLuint) * BODY_PART_COUNT);
@@ -115,17 +143,28 @@ void CharEditor(
 	LoadTexFromPic(*texid, GetHeadPic(c, DIRECTION_DOWN, false, &cc));
 	CA_FOREACH_END()
 
-	TexArrayInit(&ec.texIdsHairs, gPicManager.hairstyleNames.size);
-	CA_FOREACH(const GLuint, texid, ec.texIdsHairs)
-	const char *hair = IndexHairName(_ca_index);
-	LoadTexFromPic(*texid, GetHairPic(hair, DIRECTION_DOWN, false, &cc));
-	CA_FOREACH_END()
+	for (HeadPart hp = HEAD_PART_HAIR; hp < HEAD_PART_COUNT; hp++)
+	{
+		TexArrayInit(&ec.texIdsHeadParts[hp], gPicManager.headPartNames[hp].size);
+		CA_FOREACH(const GLuint, texid, ec.texIdsHeadParts[hp])
+		const char *name = IndexHeadPartName(_ca_index, hp);
+		LoadTexFromPic(*texid, GetHeadPartPic(name, hp, DIRECTION_DOWN, false, &cc));
+		CA_FOREACH_END()
+	}
 
 	TexArrayInit(&ec.texIdsGuns, NumGuns());
 	CA_FOREACH(const GLuint, texid, ec.texIdsGuns)
 	const WeaponClass *wc = IndexWeaponClassReal(_ca_index);
 	LoadTexFromPic(*texid, wc->Icon);
 	CA_FOREACH_END()
+	
+	TexArrayInit(&ec.texIdsMelees, NumMelees());
+	CA_FOREACH(const GLuint, texid, ec.texIdsMelees)
+	const WeaponClass *wc = IndexWeaponClassMelee(_ca_index);
+	if (wc == NULL)
+		continue;
+	LoadTexFromPic(*texid, wc->Icon);
+	CA_FOREACH_END()
 
 	TexArrayInit(&ec.texIdsPickups, NumPickups());
 	CA_FOREACH(const GLuint, texid, ec.texIdsPickups)
@@ -144,22 +183,29 @@ void CharEditor(
 	NKWindow(cfg);
 
 	CFREE(ec.CharacterClassNames);
-	CFREE(ec.HairNames);
+	for (HeadPart hp = HEAD_PART_HAIR; hp < HEAD_PART_COUNT; hp++)
+	{
+		CFREE(ec.HeadPartNames[hp]);
+	}
 	CFREE(ec.GunNames);
+	CFREE(ec.MeleeNames);
 	CFREE(ec.PickupNames);
 	glDeleteTextures(
 		(GLsizei)(BODY_PART_COUNT * ec.texidsChars.size), ec.texidsChars.data);
 	CArrayTerminate(&ec.texidsChars);
 	glDeleteTextures(BODY_PART_COUNT, ec.texidsPreview);
-	glDeleteTextures(
-		(GLsizei)ec.texIdsCharClasses.size, ec.texIdsCharClasses.data);
-	TexArrayTerminate(&ec.texIdsCharClasses);
-	glDeleteTextures((GLsizei)ec.texIdsHairs.size, ec.texIdsHairs.data);
-	TexArrayTerminate(&ec.texIdsHairs);
-	glDeleteTextures((GLsizei)ec.texIdsGuns.size, ec.texIdsGuns.data);
-	TexArrayTerminate(&ec.texIdsGuns);
-	glDeleteTextures((GLsizei)ec.texIdsPickups.size, ec.texIdsPickups.data);
-	TexArrayTerminate(&ec.texIdsPickups);
+#define DELTEX(_tx) \
+	glDeleteTextures( \
+		(GLsizei)(_tx).size, (_tx).data); \
+	TexArrayTerminate(&(_tx));
+	DELTEX(ec.texIdsCharClasses);
+	for (HeadPart hp = HEAD_PART_HAIR; hp < HEAD_PART_COUNT; hp++)
+	{
+		DELTEX(ec.texIdsHeadParts[hp]);
+	}
+	DELTEX(ec.texIdsGuns);
+	DELTEX(ec.texIdsMelees);
+	DELTEX(ec.texIdsPickups);
 }
 
 static const char *IndexCharacterClassName(const int i)
@@ -167,9 +213,9 @@ static const char *IndexCharacterClassName(const int i)
 	const CharacterClass *c = IndexCharacterClass(i);
 	return c->Name;
 }
-static const char *IndexHairName(const int i)
+static const char *IndexHeadPartName(const int i, const HeadPart hp)
 {
-	return *(char **)CArrayGet(&gPicManager.hairstyleNames, i);
+	return *(char **)CArrayGet(&gPicManager.headPartNames[hp], i);
 }
 static int NumCharacterClasses(void)
 {
@@ -181,6 +227,11 @@ static const char *IndexGunName(const int i)
 	const WeaponClass *wc = IndexWeaponClassReal(i);
 	return wc ? wc->name : NULL;
 }
+static const char *IndexMeleeName(const int i)
+{
+	const WeaponClass *wc = IndexWeaponClassMelee(i);
+	return wc ? wc->name : "(None)";
+}
 static const char *IndexPickupName(const int i)
 {
 	const PickupClass *pc = IndexPickupClass(i);
@@ -214,6 +265,38 @@ static const WeaponClass *IndexWeaponClassReal(const int i)
 	CASSERT(false, "cannot find gun");
 	return NULL;
 }
+static const WeaponClass *IndexWeaponClassMelee(const int i)
+{
+	if (i == 0)
+	{
+		return NULL;
+	}
+	int j = 1;
+	CA_FOREACH(WeaponClass, wc, gWeaponClasses.Guns)
+	if (!wc->IsRealGun || wc->Type != GUNTYPE_MELEE)
+	{
+		continue;
+	}
+	if (j == i)
+	{
+		return wc;
+	}
+	j++;
+	CA_FOREACH_END()
+	CA_FOREACH(WeaponClass, wc, gWeaponClasses.CustomGuns)
+	if (!wc->IsRealGun || wc->Type != GUNTYPE_MELEE)
+	{
+		continue;
+	}
+	if (j == i)
+	{
+		return wc;
+	}
+	j++;
+	CA_FOREACH_END()
+	CASSERT(false, "cannot find melee");
+	return NULL;
+}
 static const PickupClass *IndexPickupClass(const int i)
 {
 	if (i == 0)
@@ -262,6 +345,23 @@ static int NumGuns(void)
 	CA_FOREACH_END()
 	return total;
 }
+static int NumMelees(void)
+{
+	int total = 0;
+	CA_FOREACH(const WeaponClass, wc, gWeaponClasses.Guns)
+	if (wc->IsRealGun && wc->Type == GUNTYPE_MELEE)
+	{
+		total++;
+	}
+	CA_FOREACH_END()
+	CA_FOREACH(const WeaponClass, wc, gWeaponClasses.CustomGuns)
+	if (wc->IsRealGun && wc->Type == GUNTYPE_MELEE)
+	{
+		total++;
+	}
+	CA_FOREACH_END()
+	return total;
+}
 static int NumPickups(void)
 {
 	return (int)(gPickupClasses.Classes.size + gPickupClasses.CustomClasses.size + gPickupClasses.KeyClasses.size + 1);
@@ -274,7 +374,7 @@ static int GunIndex(const WeaponClass *wc)
 	{
 		continue;
 	}
-	if (wc == wc2)
+	if (strcmp(wc->name, wc2->name) == 0)
 	{
 		return j;
 	}
@@ -285,7 +385,7 @@ static int GunIndex(const WeaponClass *wc)
 	{
 		continue;
 	}
-	if (wc == wc2)
+	if (strcmp(wc->name, wc2->name) == 0)
 	{
 		return j;
 	}
@@ -294,6 +394,38 @@ static int GunIndex(const WeaponClass *wc)
 	CASSERT(false, "cannot find gun");
 	return -1;
 }
+static int MeleeIndex(const WeaponClass *wc)
+{
+	if (wc == NULL)
+	{
+		return 0;
+	}
+	int j = 1;
+	CA_FOREACH(const WeaponClass, wc2, gWeaponClasses.Guns)
+	if (!wc2->IsRealGun || wc2->Type != GUNTYPE_MELEE)
+	{
+		continue;
+	}
+	if (strcmp(wc->name, wc2->name) == 0)
+	{
+		return j;
+	}
+	j++;
+	CA_FOREACH_END()
+	CA_FOREACH(const WeaponClass, wc2, gWeaponClasses.CustomGuns)
+	if (!wc2->IsRealGun || wc2->Type != GUNTYPE_MELEE)
+	{
+		continue;
+	}
+	if (strcmp(wc->name, wc2->name) == 0)
+	{
+		return j;
+	}
+	j++;
+	CA_FOREACH_END()
+	CASSERT(false, "cannot find melee");
+	return -1;
+}
 static int PickupIndex(const PickupClass *pc)
 {
 	if (pc == NULL)
@@ -322,6 +454,7 @@ static int PickupIndex(const PickupClass *pc)
 	}
 	j++;
 	CA_FOREACH_END()
+	CASSERT(false, "cannot find pickup");
 	return -1;
 }
 
@@ -333,7 +466,7 @@ static int DrawClassSelection(
 	struct nk_context *ctx, EditorContext *ec, const char *label,
 	const GLuint *texids, const char *items, const int selected,
 	const size_t len);
-static int HairIndex(const char *hair);
+static int HeadPartIndex(const char *name, const HeadPart hp);
 static void DrawFlag(
 	struct nk_context *ctx, EditorContext *ec, const char *label,
 	const int flag, const char *tooltip);
@@ -499,32 +632,33 @@ static bool Draw(SDL_Window *win, struct nk_context *ctx, void *data)
 				NumCharacterClasses());
 			ec->Char->Class = IndexCharacterClass(selectedClass);
 
-			nk_layout_row_dynamic(ctx, ROW_HEIGHT, 1);
-			int hasHair = ec->Char->Hair != NULL;
-			nk_checkbox_label(ctx, "Has Hair", &hasHair);
-
-			if (hasHair)
+			for (HeadPart hp = HEAD_PART_HAIR; hp < HEAD_PART_COUNT; hp++)
 			{
-				nk_layout_row(ctx, NK_DYNAMIC, ROW_HEIGHT, 2, colRatios);
-				const int currentHair = (int)HairIndex(ec->Char->Hair);
-				int selectedHair = DrawClassSelection(
-					ctx, ec, "Hair:", ec->texIdsHairs.data, ec->HairNames,
-					currentHair, gPicManager.hairstyleNames.size);
-				if (selectedHair == -1)
+				nk_layout_row_dynamic(ctx, ROW_HEIGHT, 1);
+				int hasStyle = ec->Char->HeadParts[hp] != NULL;
+				char buf[256];
+				sprintf(buf, "Has %s", HeadPartStr(hp));
+				nk_checkbox_label(ctx, buf, &hasStyle);
+				if (hasStyle)
 				{
-					selectedHair = 0;
+					nk_layout_row(ctx, NK_DYNAMIC, ROW_HEIGHT, 2, colRatios);
+					const int current = HeadPartIndex(ec->Char->HeadParts[hp], hp);
+					sprintf(buf, "%s:", HeadPartStr(hp));
+					int selected = DrawClassSelection(ctx, ec, buf, ec->texIdsHeadParts[hp].data, ec->HeadPartNames[hp], current, gPicManager.headPartNames[hp].size);
+					if (selected == -1)
+					{
+						selected = 0;
+					}
+					if (current != selected)
+					{
+						CharacterSetHeadPart(ec->Char, hp, IndexHeadPartName(selected, hp));
+					}
 				}
-				if (currentHair != selectedHair)
+				else
 				{
-					CFREE(ec->Char->Hair);
-					CSTRDUP(ec->Char->Hair, IndexHairName(selectedHair));
+					CharacterSetHeadPart(ec->Char, hp, NULL);
 				}
 			}
-			else
-			{
-				CFREE(ec->Char->Hair);
-				ec->Char->Hair = NULL;
-			}
 
 			// Character colours
 			nk_layout_row(ctx, NK_DYNAMIC, ROW_HEIGHT, 2, colRatios);
@@ -536,6 +670,18 @@ static bool Draw(SDL_Window *win, struct nk_context *ctx, void *data)
 			{
 				*ec->FileChanged = true;
 			}
+			if (ColorPicker(ctx, ROW_HEIGHT, "Facial Hair:", &ec->Char->Colors.Facehair))
+			{
+				*ec->FileChanged = true;
+			}
+			if (ColorPicker(ctx, ROW_HEIGHT, "Hat:", &ec->Char->Colors.Hat))
+			{
+				*ec->FileChanged = true;
+			}
+			if (ColorPicker(ctx, ROW_HEIGHT, "Glasses:", &ec->Char->Colors.Glasses))
+			{
+				*ec->FileChanged = true;
+			}
 			if (ColorPicker(ctx, ROW_HEIGHT, "Arms:", &ec->Char->Colors.Arms))
 			{
 				*ec->FileChanged = true;
@@ -557,7 +703,7 @@ static bool Draw(SDL_Window *win, struct nk_context *ctx, void *data)
 
 		if (nk_begin(
 				ctx, "Attributes",
-				nk_rect(280, (float)charStoreSize.y + PAD, 250, 280),
+				nk_rect(280, (float)charStoreSize.y + PAD, 250, 310),
 				NK_WINDOW_BORDER | NK_WINDOW_TITLE))
 		{
 			// Speed (256 = 100%)
@@ -594,10 +740,16 @@ static bool Draw(SDL_Window *win, struct nk_context *ctx, void *data)
 				ctx, ec, "Gun:", ec->texIdsGuns.data, ec->GunNames,
 				GunIndex(ec->Char->Gun), NumGuns());
 			ec->Char->Gun = IndexWeaponClassReal(selectedGun);
+			const int selectedMelee = DrawClassSelection(
+				ctx, ec, "Melee:", ec->texIdsMelees.data, ec->MeleeNames,
+				MeleeIndex(ec->Char->Melee), NumMelees());
+			ec->Char->Melee = IndexWeaponClassMelee(selectedMelee);
 
 			nk_layout_row_dynamic(ctx, ROW_HEIGHT, 1);
 			nk_property_int(
 				ctx, "Max Health:", 10, &ec->Char->maxHealth, 2000, 10, 1);
+			nk_property_int(
+				ctx, "Excess Health:", ec->Char->maxHealth, &ec->Char->excessHealth, 2000, 10, 1);
 
 			nk_layout_row_dynamic(ctx, ROW_HEIGHT, 2);
 			DrawFlag(ctx, ec, "Asbestos", FLAGS_ASBESTOS, "Immune to fire");
@@ -668,6 +820,9 @@ static bool Draw(SDL_Window *win, struct nk_context *ctx, void *data)
 			DrawFlag(
 				ctx, ec, "Unlock Doors", FLAGS_UNLOCK_DOORS,
 				"Can open any locked door");
+			DrawFlag(
+				ctx, ec, "Move and shoot", FLAGS_MOVE_AND_SHOOT,
+				"Can move while shooting");
 			nk_end(ctx);
 		}
 	}
@@ -691,7 +846,11 @@ static void AddCharacter(EditorContext *ec, const int cloneIdx)
 		{
 			CSTRDUP(ec->Char->PlayerTemplateName, clone->PlayerTemplateName);
 		}
-		CSTRDUP(ec->Char->Hair, clone->Hair);
+		for (HeadPart hp = HEAD_PART_HAIR; hp < HEAD_PART_COUNT; hp++)
+		{
+			ec->Char->HeadParts[hp] = NULL;
+			CharacterSetHeadPart(ec->Char, hp, clone->HeadParts[hp]);
+		}
 		CMALLOC(ec->Char->bot, sizeof *ec->Char->bot);
 		memcpy(ec->Char->bot, clone->bot, sizeof *ec->Char->bot);
 	}
@@ -700,15 +859,18 @@ static void AddCharacter(EditorContext *ec, const int cloneIdx)
 		// set up character template
 		ec->Char->Class = StrCharacterClass("Ogre");
 		ec->Char->Colors.Skin = colorGreen;
-		const color_t darkGray = {64, 64, 64, 255};
-		ec->Char->Colors.Arms = darkGray;
-		ec->Char->Colors.Body = darkGray;
-		ec->Char->Colors.Legs = darkGray;
+		ec->Char->Colors.Arms = colorDarkGray;
+		ec->Char->Colors.Body = colorDarkGray;
+		ec->Char->Colors.Legs = colorDarkGray;
 		ec->Char->Colors.Hair = colorBlack;
-		ec->Char->Colors.Feet = darkGray;
+		ec->Char->Colors.Feet = colorDarkGray;
+		ec->Char->Colors.Facehair = colorDarkGray;
+		ec->Char->Colors.Hat = colorDarkGray;
+		ec->Char->Colors.Glasses = colorDarkGray;
 		ec->Char->speed = 1;
 		ec->Char->Gun = StrWeaponClass("Machine gun");
 		ec->Char->maxHealth = 40;
+		ec->Char->excessHealth = 80;
 		ec->Char->flags = FLAGS_IMMUNITY;
 		ec->Char->bot->probabilityToMove = 50;
 		ec->Char->bot->probabilityToTrack = 25;
@@ -786,14 +948,14 @@ static int DrawClassSelection(
 	return selectedNew;
 }
 
-static int HairIndex(const char *hair)
+static int HeadPartIndex(const char *name, const HeadPart hp)
 {
-	if (hair == NULL)
+	if (name == NULL)
 	{
 		return -1;
 	}
-	CA_FOREACH(const char *, hairstyleName, gPicManager.hairstyleNames)
-	if (strcmp(*hairstyleName, hair) == 0)
+	CA_FOREACH(const char *, hpName, gPicManager.headPartNames[hp])
+	if (strcmp(*hpName, name) == 0)
 	{
 		return _ca_index;
 	}
@@ -805,9 +967,8 @@ static void DrawFlag(
 	struct nk_context *ctx, EditorContext *ec, const char *label,
 	const int flag, const char *tooltip)
 {
-	struct nk_rect bounds = nk_widget_bounds(ctx);
 	nk_checkbox_flags_label(ctx, label, &ec->Char->flags, flag);
-	if (tooltip && nk_input_is_mouse_hovering_rect(&ctx->input, bounds))
+	if (tooltip && nk_widget_is_hovered(ctx))
 	{
 		nk_tooltip(ctx, tooltip);
 	}
