@@ -244,6 +244,11 @@ static GameLoopResult MainMenuUpdate(GameLoopData *data, LoopRunner *l)
 }
 static void MainMenuDraw(GameLoopData *data)
 {
+	if (gCampaign.IsLoaded)
+	{
+		// Loaded game already; skip menus and go straight to game
+		return;
+	}
 	MainMenuData *mData = data->Data;
 	MenuDraw(&mData->ms);
 	const struct vec2 pos =
@@ -286,7 +291,6 @@ typedef struct
 	int MenuJoinIndex;
 } CheckLANServerData;
 static menu_t *MenuCreateContinue(const char *name, MainMenuData *mainMenu, const CampaignEntry *entry);
-static menu_t *MenuCreateQuickPlay(const char *name, MainMenuData *mainMenu);
 static menu_t *MenuCreateCampaigns(
 	const char *name, const char *title, MainMenuData *mainMenu, CampaignList *list,
 	const GameMode mode);
@@ -305,8 +309,6 @@ static menu_t *MenuCreateStart(
 		menu, MenuCreateCampaigns(
 				  "Campaign", "Select a campaign:", mainMenu, &mainMenu->campaigns.campaignList,
 				  GAME_MODE_NORMAL));
-	MenuAddSubmenu(
-		menu, MenuCreateQuickPlay("Quick Play", mainMenu));
 	MenuAddSubmenu(
 		menu, MenuCreateCampaigns(
 				  "Dogfight", "Select a dogfight scenario:",
@@ -366,10 +368,6 @@ static menu_t *MenuCreateContinue(const char *name, MainMenuData *mainMenu, cons
 {
 	return CreateStartGameMode(name, mainMenu, GAME_MODE_NORMAL, entry);
 }
-static menu_t *MenuCreateQuickPlay(const char *name, MainMenuData *mainMenu)
-{
-	return CreateStartGameMode(name, mainMenu, GAME_MODE_QUICK_PLAY, &mainMenu->campaigns.quickPlayEntry);
-}
 
 static menu_t *MenuCreateCampaignItem(MainMenuData *mainMenu, CampaignEntry *entry, const GameMode mode);
 
@@ -682,14 +680,14 @@ menu_t *MenuCreateQuit(const char *name)
 static void MenuCreateKeysSingleSection(
 	menu_t *menu, const char *sectionName, const int playerIndex);
 static menu_t *MenuCreateOptionChangeKey(
-	const key_code_e code, const int playerIndex, const bool isOptional);
+	const char *name, const key_code_e code, const int playerIndex, const bool isOptional);
 
 static menu_t *MenuCreateKeys(const char *name, MainMenuData *data)
 {
 	menu_t *menu = MenuCreateNormal(name, "", MENU_TYPE_OPTIONS, 0);
 	MenuCreateKeysSingleSection(menu, "Keyboard 1", 0);
 	MenuCreateKeysSingleSection(menu, "Keyboard 2", 1);
-	MenuAddSubmenu(menu, MenuCreateOptionChangeKey(KEY_CODE_MAP, 0, true));
+	MenuAddSubmenu(menu, MenuCreateOptionChangeKey("map", KEY_CODE_MAP, 0, true));
 	MenuAddSubmenu(menu, MenuCreateSeparator(""));
 	MenuAddSubmenu(menu, MenuCreateBack("Done"));
 	MenuSetPostInputFunc(menu, PostInputConfigApply, data);
@@ -701,27 +699,31 @@ static void MenuCreateKeysSingleSection(
 {
 	MenuAddSubmenu(menu, MenuCreateSeparator(sectionName));
 	MenuAddSubmenu(
-		menu, MenuCreateOptionChangeKey(KEY_CODE_LEFT, playerIndex, false));
+		menu, MenuCreateOptionChangeKey("left", KEY_CODE_LEFT, playerIndex, false));
 	MenuAddSubmenu(
-		menu, MenuCreateOptionChangeKey(KEY_CODE_RIGHT, playerIndex, false));
+		menu,
+		MenuCreateOptionChangeKey("right", KEY_CODE_RIGHT, playerIndex, false));
 	MenuAddSubmenu(
-		menu, MenuCreateOptionChangeKey(KEY_CODE_UP, playerIndex, false));
+		menu, MenuCreateOptionChangeKey("up", KEY_CODE_UP, playerIndex, false));
 	MenuAddSubmenu(
-		menu, MenuCreateOptionChangeKey(KEY_CODE_DOWN, playerIndex, false));
+		menu, MenuCreateOptionChangeKey("down", KEY_CODE_DOWN, playerIndex, false));
 	MenuAddSubmenu(
-		menu, MenuCreateOptionChangeKey(KEY_CODE_BUTTON1, playerIndex, false));
+		menu,
+		MenuCreateOptionChangeKey("fire", KEY_CODE_BUTTON1, playerIndex, false));
 	MenuAddSubmenu(
-		menu, MenuCreateOptionChangeKey(KEY_CODE_BUTTON2, playerIndex, false));
+		menu,
+		MenuCreateOptionChangeKey("switch/strafe", KEY_CODE_BUTTON2, playerIndex, false));
 	MenuAddSubmenu(
-		menu, MenuCreateOptionChangeKey(KEY_CODE_GRENADE, playerIndex, true));
+		menu,
+		MenuCreateOptionChangeKey("grenade", KEY_CODE_GRENADE, playerIndex, true));
 	MenuAddSubmenu(menu, MenuCreateSeparator(""));
 }
 
 static menu_t *MenuCreateOptionChangeKey(
-	const key_code_e code, const int playerIndex, const bool isOptional)
+	const char *name, const key_code_e code, const int playerIndex, const bool isOptional)
 {
 	menu_t *menu =
-		MenuCreate(KeycodeStr(code), MENU_TYPE_SET_OPTION_CHANGE_KEY);
+		MenuCreate(name, MENU_TYPE_SET_OPTION_CHANGE_KEY);
 	menu->u.changeKey.code = code;
 	menu->u.changeKey.playerIndex = playerIndex;
 	menu->u.changeKey.isOptional = isOptional;
